#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2024, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

"""Ansible module to perform Network Compliance Operations on devices in Cisco Catalyst Center."""
from __future__ import absolute_import, division, print_function

__metaclass__ = type
__author__ = ("Rugvedi Kapse, Madhan Sankaranarayanan, Sonali Deepthi Kesali")

DOCUMENTATION = r"""
---
module: network_compliance_workflow_manager
short_description: Network Compliance module for managing network compliance tasks on reachable device(s) in Cisco Catalyst Center.
description:
- Perform compliance checks or sync configurations on reachable devices using IP Address(s) or Site.
- API to perform full compliance checks or specific category checks on reachable device(s).
- API to sync device configuration on device(s).
version_added: "6.14.0"
extends_documentation_fragment:
  - cisco.dnac.workflow_manager_params
author: Rugvedi Kapse (@rukapse)
        Madhan Sankaranarayanan (@madhansansel)
        Sonali Deepthi (@skesali)
options:
  config_verify:
    description: Set to True to verify the Cisco Catalyst Center config after applying the playbook config.
    type: bool
    default: False
  state:
    description: State of Cisco Catalyst Center after module completion.
    type: str
    choices: [ merged ]
    default: merged
  config:
    description: List of device details for running a compliance check or synchronizing device configuration.
    type: list
    elements: dict
    required: True
    suboptions:
      ip_address_list:
        description: List of IP addresses of devices to run a compliance check on or synchronize device configurations.
                     Either "ip_address_list" or "site_name" is required for module to execute.
                     If both "site_name" and "ip_address_list" are provided, operations are performed on devices that are present in both the
                     "ip_address_list" and the specified site.
                     (e.g. ["204.1.2.2", "204.1.2.5", "204.1.2.4"])
        type: list
        elements: str
      site_name:
        description: When "site_name" is specified, the module executes the operation on all the devices located within the specified site.
                     This is a string value that should represent the complete hierarchical path of the site.
                     Either "site_name" or "ip_address_list" is required for module to execute.
                     If both "site_name" and "ip_address_list" are provided, operations are performed on devices that are present in both the
                     "ip_address_list" and the specified site.
                     (e.g. "Global/USA/San Francisco/Building_2/floor_1")
        type: str
      run_compliance:
        description: Determines if a full compliance check should be triggered on the devices specified in the "ip_address_list" and/or "site_name".
                     if it is True then compliance will be triggered for all categories.
                     If it is False then compliance will be not be triggered even if run_compliance categories are provided.
                     Note - This operation cannot be performed on Access Points (APs) and if APs are provided, they will be skipped.
        type: bool
        default: True
      run_compliance_batch_size:
        description: Specifies the number of devices to be included in a single batch for compliance operations.
                     This parameter is crucial for optimizing performance during large-scale compliance checks.
                     By processing devices in manageable batches, the system can enhance the speed and efficiency of the operation,
                     reducing the overall time required and minimizing the risk of overloading system resources.
                     Adjusting this parameter allows for a balance between throughput and resource utilization, ensuring smooth and
                     effective compliance management.
                     Note - Having a higher value for run_compliance_batch_size may cause errors due to the increased load on the system.
        type: int
        default: 100
      run_compliance_categories:
        description: Specifying compliance categories allows you to trigger compliance checks only for the mentioned categories.
                     Category can have one or more values from among the options "INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT", "EOX", "NETWORK_SETTINGS".
                     Category "INTENT" is mapped to compliance types "NETWORK_SETTINGS", "NETWORK_PROFILE", "WORKFLOW", "FABRIC", "APPLICATION_VISIBILITY".
                     If "run_compliance" is False then compliance will be not be triggered even if "run_compliance_categories" are provided.
                     (e.g. ["INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT", "EOX", "NETWORK_SETTINGS"])
        type: list
        elements: str
      sync_device_config:
        description: Determines whether to synchronize the device configuration on the devices specified in the "ip_address_list" and/or "site_name".
                     Sync device configuration, primarily addresses the status of the `RUNNING_CONFIG`.
                     If set to True, and if `RUNNING_CONFIG` status is non-compliant this operation would commit device running configuration
                     to startup by issuing "write memory" to device.
                     Note - This operation cannot be performed on Access Points (APs) and if APs are provided, they will be skipped.
        type: bool
        default: False

requirements:
- dnacentersdk == 2.7.0
- python >= 3.9
notes:
  - SDK Methods used are
    compliance.Compliance.run_compliance
    compliance.Compliance.commit_device_configuration
    task.Task.get_task_by_id
    task.Task.get_task_details_by_id
    task.Task.get_tasks
    compliance.Compliance.compliance_details_of_device
    devices.Devices.get_device_list
    devices.Devices.get_device_by_id
    site.Site.get_site
    site.Site.get_membership
    site_design.Site_design.get_sites
    site_design.Site_design.get_site_assigned_network_devices

  - Paths used are
    post /dna/intent/api/v1/compliance/
    post /dna/intent/api/v1/network-device-config/write-memory
    get /dna/intent/api/v1/task/{taskId}
    get /dna/intent/api/v1/compliance/${deviceUuid}/detail
    get /dna/intent/api/v1/membership/${siteId}
    get /dna/intent/api/v1/site
    get /dna/intent/api/v1/networkDevices/assignedToSite
    get /dna/intent/api/v1/sites
    get /dna/intent/api/v1/tasks/${id}/detail
    get /dna/intent/api/v1/tasks
    get /dna/intent/api/v1/network-device/${id}
    get /dna/intent/api/v1/network-device

"""

EXAMPLES = r"""
- name: Run Compliance check on device(s) using IP address list (run_compliance by default is True)
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]

- name: Run Compliance check on device(s) using IP address list
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]
        run_compliance: true

- name: Run Compliance check on device(s) using Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - site_name: "Global/USA/San Francisco/Building_1/floor_1"
        run_compliance: true

- name: Run Compliance check on device(s) using both IP address list and Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]
        site_name: "Global/USA/San Francisco/Building_1/floor_1"
        run_compliance: true

- name: Run Compliance check with specific categories on device(s) using IP address list
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]
        run_compliance: true
        run_compliance_categories: ["INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT"]

- name: Run Compliance check with specific categories on device(s) using Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - site_name: "Global/USA/San Francisco/Building_1/floor_1"
        run_compliance: true
        run_compliance_categories: ["INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT"]

- name: Run Compliance check with specific categories on device(s) using both IP address list and Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]
        site_name: "Global/USA/San Francisco/Building_1/floor_1"
        run_compliance: true
        run_compliance_categories: ["INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT"]

- name: Sync Device Configuration on device(s) using IP address list
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - site_name: "Global"
        sync_device_config: true
        run_compliance: False

- name: Sync Device Configuration on device(s) using Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - site_name: "Global/USA/San Francisco/Building_1/floor_1"
        sync_device_config: true
        run_compliance: false

- name: Sync Device Configuration on device(s) using both IP address list and Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]
        site_name: "Global/USA/San Francisco/Building_1/floor_1"
        sync_device_config: true
        run_compliance: false

- name: Run Compliance and Sync Device Configuration using both IP address list and Site
  cisco.dnac.network_compliance_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: false
    config:
      - ip_address_list: ["204.1.2.2", "204.1.2.5", "204.1.2.4"]
        site_name: "Global/USA/San Francisco/Building_1/floor_1"
        run_compliance: true
        run_compliance_categories: ["INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT"]
        sync_device_config: true
"""

RETURN = r"""
#Case_1: Response when Run Compliance operation is performed successfully on device/s.
sample_response_1:
  description: A dictionary with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "changed": bool,
      "msg": "string"
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "data": dict,
      "version": "string"
    }

#Case_2: Response when Sync Device Configuration operation is performed successfully on device/s.
sample_response_2:
  description: A dictionary with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "changed": bool,
      "msg": "string"
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }

#Case_3: Response when Error Occurs in performing Run Compliance or Sync Device Configuration operation on device/s.
sample_response_3:
  description: A dictionary with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: dict
  sample: >
    {
      "changed": bool,
      "msg": "string"
    }
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.dnac.plugins.module_utils.dnac import (
    DnacBase,
    validate_list_of_dicts
)


class NetworkCompliance(DnacBase):
    """Class containing member attributes for network_compliance_workflow_manager module"""

    def __init__(self, module):
        """
        Initialize an instance of the class.
        Args:
          - module: The module associated with the class instance.
        Returns:
          The method does not return a value.
        """
        super().__init__(module)
        self.skipped_run_compliance_devices_list = []
        self.skipped_sync_device_configs_list = []

    def validate_input(self):
        """
        Validate the fields provided in the playbook against a predefined specification
        to ensure they adhere to the expected structure and data types.
        Args:
            state (optional): A state parameter that can be used to customize validation
                              based on different conditions.
        Returns:
            object: An instance of the class with updated attributes:
              - self.msg: A message describing the validation result.
              - self.status: The status of the validation (either "success" or "failed").
              - self.validated_config: If successful, a validated version of the "config" parameter.
        Description:
            This method validates the fields provided in the playbook against a predefined specification.
            It checks if the required fields are present and if their data types match the expected types.
            If any parameter is found to be invalid, it logs an error message and sets the validation status to "failed".
            If the validation is successful, it logs a success message and returns an instance of the class
            with the validated configuration.
        """
        if not self.config:
            self.msg = "config not available in playbook for validation"
            self.status = "success"
            self.log(self.msg, "ERROR")
            return self

        temp_spec = {
            "ip_address_list": {"type": "list", "elements": "str", "required": False},
            "site_name": {"type": "str", "required": False},
            "run_compliance": {"type": "bool", "required": False, "default": True},
            "run_compliance_categories": {"type": "list", "elements": "str", "required": False},
            "run_compliance_batch_size": {"type": "int", "required": False, "default": 100},
            "sync_device_config": {"type": "bool", "required": False, "default": False},
        }

        # Validate device params
        valid_temp, invalid_params = validate_list_of_dicts(
            self.config, temp_spec
        )

        if invalid_params:
            self.msg = "Invalid parameters in playbook: {0}".format(invalid_params)
            self.log(self.msg, "ERROR")
            self.status = "failed"
            return self

        self.validated_config = valid_temp

        self.msg = "Successfully validated playbook configuration parameters using 'validated_input': {0}".format(str(valid_temp))
        self.log(self.msg, "INFO")
        self.status = "success"

        return self

    def validate_ip4_address_list(self, ip_address_list):
        """
        Validates the list of IPv4 addresses provided in the playbook.
        Args:
            ip_address_list (list): A list of IPv4 addresses to be validated.
        Description:
            This method iterates through each IP address in the list and checks if it is a valid IPv4 address.
            If any address is found to be invalid, it logs an error message and fails.
            After validating all IP addresses, it logs a success message.
        """
        self.log("Validating the IP addresses in the ip_address_list: {0}".format(ip_address_list), "DEBUG")

        for ip in ip_address_list:
            if not self.is_valid_ipv4(ip):
                self.msg = "IP address: {0} is not valid".format(ip)
                self.log(self.msg, "ERROR")
                self.module.fail_json(self.msg)

        ip_address_list_str = ", ".join(ip_address_list)
        self.log("Successfully validated the IP address(es): {0}".format(ip_address_list_str), "DEBUG")

    def validate_iplist_and_site_name(self, ip_address_list, site_name):
        """
        Validates that either an IP address list or a site name is provided.
        This function checks if at least one of the parameters `ip_address_list` or `site_name` is provided.
        If neither is provided, it logs an error message and exits the process. If validation is successful,
        it logs a success message.
        Args:
            ip_address_list (list): A list of IP addresses to be validated.
            site_name (str): A site name to be validated.
        Raises:
            SystemExit: If neither `ip_address_list` nor `site_name` is provided, the function logs an error message and exits the process.
        """
        self.log("Validating 'ip_address_list': '{0}' or 'site_name': '{1}'".format(ip_address_list, site_name), "DEBUG")

        # Check if IP address list or hostname is provided
        if not any([ip_address_list, site_name]):
            self.msg = "Error: Neither 'ip_address_list' nor 'site_name' was provided. Provided values: 'ip_address_list': {0}, 'site_name': {1}.".format(
                ip_address_list, site_name)
            self.fail_and_exit(self.msg)

        # Validate if valid ip_addresses in the ip_address_list
        if ip_address_list:
            self.validate_ip4_address_list(ip_address_list)

        self.log("Validation successful: Provided IP address list or Site name is valid")

    def validate_compliance_operation(self, run_compliance, run_compliance_categories, sync_device_config):
        """
        Validates if any network compliance operation is requested.
        Args:
            run_compliance (bool): Indicates if a compliance check operation is requested.
            run_compliance_categories (list): A list of compliance categories to be checked.
            sync_device_config (bool): Indicates if a device configuration synchronization is requested.
        Raises:
            Exception: If no compliance operation is requested, raises an exception with a message.
        """
        self.log(
            "Validating if any network compliance operation is requested: "
            "run_compliance={0}, run_compliance_categories={1}, sync_device_config={2}".format(
                run_compliance, run_compliance_categories, sync_device_config
            ),
            "DEBUG"
        )

        if not any([run_compliance, run_compliance_categories, sync_device_config]):
            self.msg = (
                "No actions were requested. This network compliance module can perform the following tasks: "
                "Run Compliance Check or Sync Device Config."
            )
            self.fail_and_exit(self.msg)

        self.log("Validation successful: Network Compliance operation present")

    def validate_run_compliance_categories(self, run_compliance_categories):
        """
        Validates the provided Run Compliance categories.
        Args:
        run_compliance_categories (list): A list of compliance categories to be checked.
        Raises:
            Exception: If invalid categories are provided, raises an exception with a message.
        """
        self.log("Validating the provided run compliance categories: {0}".format(run_compliance_categories), "DEBUG")

        valid_categories = ["INTENT", "RUNNING_CONFIG", "IMAGE", "PSIRT", "EOX", "NETWORK_SETTINGS"]
        if not all(category.upper() in valid_categories for category in run_compliance_categories):
            valid_categories_str = ", ".join(valid_categories)
            self.msg = "Invalid category provided. Valid categories are {0}.".format(valid_categories_str)
            self.fail_and_exit(self.msg)

        self.log("Validation successful: valid run compliance categorites provided: {0}".format(run_compliance_categories), "DEBUG")

    def validate_params(self, config):
        """
        Validates the provided configuration for network compliance operations.
        Args:
            config (dict): A dictionary containing the configuration parameters.
        Validations:
            - Ensures that either ip_address_list or site_name is provided.
            - Checks if a network compliance operation is requested.
            - Validates the compliance categories if provided.
        Raises:
            Exception: If any validation fails, raises an exception with a message.
        """
        self.log("Validating the provided configuration: {0}".format(config), "INFO")
        ip_address_list = config.get("ip_address_list")
        site_name = config.get("site_name")
        run_compliance = config.get("run_compliance")
        run_compliance_categories = config.get("run_compliance_categories")
        sync_device_config = config.get("sync_device_config")
        self.log(
            "Extracted parameters - IP Address List: {0}, Site Name: {1}, Run Compliance: {2}, "
            "Run Compliance Categories: {3}, Sync Device Config: {4}".format(
                ip_address_list, site_name, run_compliance, run_compliance_categories, sync_device_config
            ),
            "DEBUG"
        )

        # Validate either ip_address_list OR site_name is present
        self.validate_iplist_and_site_name(ip_address_list, site_name)

        # Validate if a network compliance operation is present
        self.validate_compliance_operation(run_compliance, run_compliance_categories, sync_device_config)

        # Validate the categories if provided
        if run_compliance_categories:
            self.validate_run_compliance_categories(run_compliance_categories)

        self.log("Validation completed for configuration: {0}".format(config), "INFO")

    def get_run_compliance_params(self, mgmt_ip_to_instance_id_map, run_compliance, run_compliance_categories):
        """
        Validate and prepare parameters for running compliance checks.
        Args:
            - mgmt_ip_to_instance_id_map (dict): A dictionary mapping management IP addresses to device instance IDs.
            - run_compliance (bool or None): A boolean indicating whether to run compliance checks.
            - run_compliance_categories (list): A list of compliance categories to check.
        Returns:
        tuple: A tuple containing two dictionaries:
            - run_compliance_params: Parameters for running compliance checks.
        Notes:
            - This method prepares parameters for running compliance checks based on the provided inputs.
            - If invalid categories are provided in `run_compliance_categories`, a `ValueError` is raised.
            - If `run_compliance_categories` is provided and neither `run_compliance` nor `run_compliance_categories` is set, an error
              is logged and the method fails.
            - If `run_compliance` is set and `run_compliance_categories` is not, full compliance checks are triggered.
            - If both `run_compliance` and `run_compliance_categories` are set, compliance checks are triggered for specific categories.
        """
        # Initializing empty dicts/lists
        run_compliance_params = {}

        # Create run_compliance_params
        if run_compliance:
            run_compliance_params["deviceUuids"] = list(mgmt_ip_to_instance_id_map.values())
            run_compliance_params["triggerFull"] = not bool(run_compliance_categories)
            if run_compliance_categories:
                run_compliance_params["categories"] = run_compliance_categories

        # Check for devices with Compliance Status of "IN_PROGRESS" and update parameters accordingly
        if run_compliance_params:
            device_in_progress = set()

            response = self.get_compliance_report(run_compliance_params, mgmt_ip_to_instance_id_map)

            if not response:
                ip_address_list_str = ", ".join(list(mgmt_ip_to_instance_id_map.keys()))
                self.msg = (
                    "Error occurred when retrieving Compliance Report to identify if there are "
                    "devices with 'IN_PROGRESS' status. This is required on device(s): {0}"
                    .format(ip_address_list_str)
                )
                self.fail_and_exit(self.msg)

            # Iterate through the response to identify devices with 'IN_PROGRESS' status
            for device_ip, compliance_details_list in response.items():
                for compliance_type in compliance_details_list:
                    if compliance_type.get("status") == "IN_PROGRESS":
                        device_in_progress.add(compliance_type.get("deviceUuid"))

            self.log(
                "Number of devices with Compliance Status 'IN_PROGRESS': {0}. Device UUIDs: {1}".format(
                    len(device_in_progress), list(device_in_progress)), "DEBUG"
            )
            if device_in_progress:
                # Update run_compliance_params to exclude devices with 'IN_PROGRESS' status
                run_compliance_params["deviceUuids"] = [device_id for device_id in mgmt_ip_to_instance_id_map.values() if device_id not in device_in_progress]
                msg = "Excluding 'IN_PROGRESS' devices from compliance check. Updated run_compliance_params: {0}".format(run_compliance_params)
                self.log(msg, "DEBUG")

        self.log("run_compliance_params: {0}".format(run_compliance_params), "DEBUG")
        return run_compliance_params

    def get_sync_device_config_params(self, mgmt_ip_to_instance_id_map, categorized_devices):
        """
        Generates parameters for syncing device configurations, excluding compliant and other categorized devices.
        Args:
            mgmt_ip_to_instance_id_map (dict): A dictionary mapping management IP addresses to instance IDs of devices.
            categorized_devices (dict): A dictionary categorizing devices by their compliance status.
        Returns:
            dict: A dictionary containing the device IDs to be used for syncing device configurations.
        Description:
            This method generates a dictionary of parameters required for syncing device configurations. It initially includes all device
            IDs from `mgmt_ip_to_instance_id_map`. It then excludes devices categorized as "OTHER" or "COMPLIANT" from the sync operation.
            The excluded devices' IPs are logged and added to the `skipped_sync_device_configs_list`. The updated list of device IDs to be synced
            is returned.
        """
        self.log("Entering get_sync_device_config_params method with mgmt_ip_to_instance_id_map: {0}, categorized_devices: {1}".format(
            mgmt_ip_to_instance_id_map, categorized_devices), "DEBUG"
        )

        sync_device_config_params = {
            "deviceId": list(mgmt_ip_to_instance_id_map.values())
        }

        other_device_ips = categorized_devices.get("OTHER", {}).keys()
        compliant_device_ips = categorized_devices.get("COMPLIANT", {}).keys()
        excluded_device_ips = set(other_device_ips) | set(compliant_device_ips)

        self.log("Identified other device IPs: {0}".format(", ".join(other_device_ips)), "DEBUG")
        self.log("Identified compliant device IPs: {0}".format(", ".join(compliant_device_ips)), "DEBUG")
        self.log("Identified excluded device IPs: {0}".format(", ".join(excluded_device_ips)), "DEBUG")

        if excluded_device_ips:
            self.skipped_sync_device_configs_list.extend(excluded_device_ips)
            excluded_device_uuids = [mgmt_ip_to_instance_id_map[ip] for ip in excluded_device_ips if ip in mgmt_ip_to_instance_id_map]
            sync_device_config_params["deviceId"] = [
                device_id for device_id in mgmt_ip_to_instance_id_map.values()
                if device_id not in excluded_device_uuids
            ]
            excluded_device_ips_str = ", ".join(excluded_device_ips)
            msg = "Skipping these devices because their compliance status is not 'NON_COMPLIANT': {0}".format(excluded_device_ips_str)
            self.log(msg, "WARNING")
            self.log("Updated 'sync_device_config_params' parameters: {0}".format(sync_device_config_params), "DEBUG")

        self.log("Final sync_device_config_params: {0}".format(sync_device_config_params), "DEBUG")
        return sync_device_config_params

    def get_device_list_params(self, ip_address_list):
        """
        Generates a dictionary of device parameters for querying Cisco Catalyst Center.
        Args:
            config (dict): A dictionary containing device filter criteria.
        Returns:
            dict: A dictionary mapping internal parameter names to their corresponding values from the config.
        Description:
            This method takes a configuration dictionary containing various device filter criteria and maps them to the internal parameter
            names required by Cisco Catalyst Center.
            It returns a dictionary of these mapped parameters which can be used to query devices based on the provided filters.
        """
        self.log("Entering get_device_list_params method with ip_address_list: {0}".format(ip_address_list), "DEBUG")

        # Initialize an empty dictionary to store the mapped parameters
        get_device_list_params = {"management_ip_address": ip_address_list}

        self.log("Generated get_device_list_params: {0}".format(get_device_list_params), "DEBUG")
        return get_device_list_params

    def get_device_ids_from_ip(self, get_device_list_params):
        """Retrieves device IDs based on specified parameters from Cisco Catalyst Center.
        Args:
            get_device_list_params (dict): A dictionary of parameters to filter devices.
        Returns:
            dict: A dictionary mapping management IP addresses to instance IDs of reachable devices that are not Unified APs.
        Description:
            This method queries Cisco Catalyst Center to retrieve device information based on the provided filter parameters.
            It paginates through the results, filters out unreachable devices and Unified APs, and returns a dictionary of management IP addresses
            mapped to their instance IDs.
            Logs detailed information about the number of devices processed, skipped, and the final list of devices available for configuration backup.
        """
        self.log("Entering 'get_device_ids_from_ip' method with parameters: {0}".format(get_device_list_params), "DEBUG")

        mgmt_ip_to_instance_id_map = {}
        processed_device_count = 0
        skipped_device_count = 0

        try:
            offset = 1
            limit = 500
            while True:
                # Update params with current offset and limit
                get_device_list_params.update({
                    "offset": offset,
                    "limit": limit
                })

                # Query Cisco Catalyst Center for device information using the parameters
                response = self.dnac._exec(
                    family="devices",
                    function="get_device_list",
                    op_modifies=False,
                    params=get_device_list_params
                )
                self.log("Response received post 'get_device_list' API call with offset {0}: {1}".format(offset, str(response)), "DEBUG")

                # Check if a valid response is received
                if not response.get("response"):
                    self.log("Exiting the loop because no devices were returned after increasing the offset. Current offset: {0}".format(offset))
                    break  # Exit loop if no devices are returned

                # Iterate over the devices in the response
                for device_info in response.get("response", []):
                    processed_device_count += 1
                    device_ip = device_info.get("managementIpAddress", "Unknown IP")
                    reachability_status = device_info.get("reachabilityStatus")
                    collection_status = device_info.get("collectionStatus")
                    device_family = device_info.get("family")
                    device_id = device_info.get("id")

                    self.log(
                        "Processing device with IP: {0}, Reachability: {1}, Collection Status: {2}, Family: {3}".format(
                            device_ip,
                            reachability_status,
                            collection_status,
                            device_family
                        ),
                        "DEBUG"
                    )
                    # Check if the device is reachable and managed
                    if reachability_status == "Reachable" and collection_status == "Managed":
                        # Skip Unified AP devices
                        if device_family != "Unified AP" :
                            mgmt_ip_to_instance_id_map[device_ip] = device_id
                        else:
                            skipped_device_count += 1
                            self.skipped_run_compliance_devices_list.append(device_ip)
                            self.skipped_sync_device_configs_list.append(device_ip)
                            msg = (
                                "Skipping device {0} as its family is: {1}.".format(
                                    device_ip, device_family
                                )
                            )
                            self.log(msg, "INFO")

                    else:
                        self.skipped_run_compliance_devices_list.append(device_ip)
                        self.skipped_sync_device_configs_list.append(device_ip)
                        skipped_device_count += 1
                        msg = (
                            "Skipping device {0} as its status is {1} or its collectionStatus is {2}.".format(
                                device_ip, reachability_status, collection_status
                            )
                        )
                        self.log(msg, "INFO")

                # Increment offset for next batch
                offset += limit

            # Check if the IP from get_device_list_params is in mgmt_ip_to_instance_id_map
            for ip in get_device_list_params.get("management_ip_address", []):
                if ip not in mgmt_ip_to_instance_id_map:
                    self.skipped_run_compliance_devices_list.append(ip)
                    self.skipped_sync_device_configs_list.append(ip)

            # Log the total number of devices processed and skipped
            self.log("Total number of devices received: {0}".format(processed_device_count), "INFO")
            self.log("Number of devices that are Unreachable or APs: {0}".format(skipped_device_count), "INFO")
            self.log("Config Backup Operation can be performed on the following filtered devices: {0}".format(len(mgmt_ip_to_instance_id_map)), "INFO")

        except Exception as e:
            # Log an error message if any exception occurs during the process
            self.log("Error fetching device IDs from Cisco Catalyst Center. Error details: {0}".format(str(e)), "ERROR")

        # Log an error if no reachable devices are found
        if not mgmt_ip_to_instance_id_map:
            self.log("No reachable devices found among the provided parameters: {0}".format(mgmt_ip_to_instance_id_map), "ERROR")

        return mgmt_ip_to_instance_id_map

    def get_device_id_list(self, ip_address_list, site_name):
        """
        Get the list of unique device IDs for a specified list of management IP addresses or devices associated with a site
        in Cisco Catalyst Center.
        Args:
            ip_address_list (list): The management IP addresses of devices for which you want to retrieve the device IDs.
            site_name (str): The name of the site for which you want to retrieve the device IDs.
        Returns:
            dict: A dictionary mapping management IP addresses to device IDs for the specified devices.
        Description:
            This method queries Cisco Catalyst Center to retrieve the unique device IDs associated with devices having the
            specified IP addresses or belonging to the specified site.
        """
        self.log("Entering get_device_id_list with args: ip_address_list={0}, site_name={1}".format(
            ip_address_list, site_name), "DEBUG")

        # Initialize a dictionary to store management IP addresses and their corresponding device IDs
        mgmt_ip_to_instance_id_map = {}

        if ip_address_list:
            self.log("Retrieving device IDs for IP addresses: {0}".format(", ".join(ip_address_list)), "DEBUG")
            get_device_list_params = self.get_device_list_params(ip_address_list)
            iplist_mgmt_ip_to_instance_id_map = self.get_device_ids_from_ip(get_device_list_params)
            mgmt_ip_to_instance_id_map.update(iplist_mgmt_ip_to_instance_id_map)

        # Check if both site name and IP address list are provided
        if site_name:
            self.log("Retrieving device IDs for site: {0}".format(site_name), "DEBUG")
            site_mgmt_ip_to_instance_id_map, skipped_devices_list = self.get_reachable_devices_from_site(site_name)
            self.skipped_run_compliance_devices_list.extend(skipped_devices_list)
            self.skipped_sync_device_configs_list.extend(skipped_devices_list)
            mgmt_ip_to_instance_id_map.update(site_mgmt_ip_to_instance_id_map)

        if not mgmt_ip_to_instance_id_map:
            # Log an error message if mgmt_ip_to_instance_id_map is empty
            ip_address_list_str = ", ".join(ip_address_list)
            self.msg = (
                "No device UUIDs were fetched for network compliance operations with the provided IP address(es): {0} "
                "or site name: {1}. This could be due to Unreachable devices or access points (APs)."
            ).format(ip_address_list_str, site_name)
            self.fail_and_exit(self.msg)

        return mgmt_ip_to_instance_id_map

    def is_sync_required(self, response, mgmt_ip_to_instance_id_map):
        """
        Determine if synchronization of device configurations is required.

        Args:
            response (dict): A dictionary containing modified responses for each device.
            mgmt_ip_to_instance_id_map (dict): A dictionary mapping management IP addresses to instance IDs.

        Returns:
            tuple: A tuple containing a boolean indicating whether synchronization is required
                   and a message explaining the result.

        Note:
            This method categorizes devices based on compliance status ("COMPLIANT", "NON_COMPLIANT", "OTHER")
            and checks if synchronization is necessary. If all devices are "COMPLIANT", synchronization is not
            required. If there are devices that are not "NON_COMPLIANT", synchronization is also not required.
        """
        task_name = "Sync Device Configuration"
        required = True
        msg = None

        # Validate if sync is required
        self.log("Compliance Report for {0} operation for device(s) {1} : {2}".format(
            task_name, list(mgmt_ip_to_instance_id_map.keys()), response), "INFO")

        # Categorize the devices based on status - "COMPLIANT", "NON_COMPLIANT", "OTHER"(status other than COMPLIANT and NON_COMPLIANT)
        categorized_devices = {"COMPLIANT": {}, "NON_COMPLIANT": {}, "OTHER": {}}
        for ip_address, compliance_type in response.items():
            status = compliance_type[0]["status"]
            if status == "NON_COMPLIANT":
                categorized_devices["NON_COMPLIANT"][ip_address] = compliance_type
            elif status == "COMPLIANT":
                categorized_devices["COMPLIANT"][ip_address] = compliance_type
            else:
                categorized_devices["OTHER"][ip_address] = compliance_type

        self.log("Device(s) Categorized based on Compliance status: {0}".format(categorized_devices), "INFO")

        # Validate if all devices are "COMPLIANT" - then sync not required
        if len(categorized_devices["COMPLIANT"]) + len(categorized_devices["OTHER"]) == len(mgmt_ip_to_instance_id_map):
            compliant_device_ips_str = ", ".join(list(mgmt_ip_to_instance_id_map.keys()))
            msg = (
                "Device(s) with IP address(es): {0} are already compliant with the RUNNING_CONFIG compliance type. "
                "Therefore, the task '{1}' is not required."
            ).format(compliant_device_ips_str, task_name)
            required = False

        return required, msg, categorized_devices

    def get_compliance_details_of_device(self, compliance_details_of_device_params, device_ip):
        """
        Retrieve compliance details for a specific device.
        This function makes an API call to fetch compliance details for a given device
        using the specified parameters. It handles the API response and logs the
        necessary information.
        Args:
            compliance_details_of_device_params (dict): Parameters required for the compliance details API call.
            device_ip (str): The IP address of the device for which compliance details are being fetched.
        Returns:
            dict or None: The response from the compliance details API call if successful,
                          None if an error occurs or no response is received.
        """
        self.log("Attempting to retrieve Compliance details for device: '{0}'".format(device_ip), "INFO")
        response = self.execute_get_request("compliance", "compliance_details_of_device", compliance_details_of_device_params)
        if response:
            self.log("Sucessfully retrieved Compliance details for device: '{0}'".format(device_ip), "INFO")
            return response.get("response")
        else:
            self.log(
                "No Compliance details retrieved for device: '{0}' with parameters: {1}".format(
                    device_ip, compliance_details_of_device_params
                ),
                "WARNING"
            )
            return None

    def get_compliance_report(self, run_compliance_params, mgmt_ip_to_instance_id_map):
        """
        Generate a compliance report for devices based on provided parameters.
        This function fetches the compliance details for a list of devices specified
        in the run_compliance_params. It maps the device UUIDs to their corresponding
        management IPs, and then retrieves the compliance details for each device.
        Args:
            run_compliance_params (dict): Parameters for running compliance checks.
                                          Expected to contain "deviceUuids" and optionally "categories".
            mgmt_ip_to_instance_id_map (dict): Mapping of device management IPs to device UUIDs.

        Returns:
            dict: A dictionary with device management IPs as keys and lists of compliance details as values.
        """
        # Initialize the lists/dicts
        final_response = {}
        device_list = []
        compliance_details_of_device_params = {}
        device_ip = None

        # Iterate through each device UUID in the run compliance parameters
        for device_uuid in run_compliance_params["deviceUuids"]:

            # Find the corresponding device IP for the given device UUID
            for ip, device_id in mgmt_ip_to_instance_id_map.items():
                if device_uuid == device_id:
                    device_ip = ip
                    break

            if device_ip is None:
                self.log("Device UUID: {0} not found in mgmt_ip_to_instance_id_map: {1}".format(device_uuid, mgmt_ip_to_instance_id_map), "DEBUG")
                continue

            # Add the device IP to the device list
            device_list.append(device_ip)

            # Initialize the response list for the device IP if not already present
            if device_ip not in final_response.keys():
                final_response[device_ip] = []

            # Check if categories are specified and fetch details for each category of the device
            if "categories" in run_compliance_params.keys():
                for category in run_compliance_params["categories"]:
                    compliance_details_of_device_params["category"] = category
                    compliance_details_of_device_params["device_uuid"] = device_uuid
                    compliance_details_of_device_params["diff_list"] = True

                    response = self.get_compliance_details_of_device(compliance_details_of_device_params, device_ip)
                    if response:
                        final_response[device_ip].extend(response)

            else:
                # Fetch compliance details for the device without specific category
                compliance_details_of_device_params["device_uuid"] = device_uuid
                compliance_details_of_device_params["diff_list"] = True
                response = self.get_compliance_details_of_device(compliance_details_of_device_params, device_ip)
                if response:
                    final_response[device_ip].extend(response)

        # If no compliance details were found, update the result with an error message
        if not final_response:
            device_list_str = ", ".join(device_list)
            self.msg = "No Compliance Details found for the devices: {0}".format(device_list_str)
            self.fail_and_exit(self.msg)

        return final_response

    def run_compliance(self, run_compliance_params, batch_size):
        """
        Executes a compliance check operation in Cisco Catalyst Center.
        Args:
            run_compliance_params (dict): Parameters for running the compliance check.
            batch_size (int): The number of devices to include in each batch.
        Returns:
            batches_dict: A dictionary containing task IDs and parameters for each batch, or an empty dictionary if unsuccessful
        Description:
            This method initiates a compliance check operation in Cisco Catalyst Center by calling the "run_compliance" function
            from the "compliance" family of APIs. It passes the provided parameters and updates the result accordingly.
        """
        # Execute the compliance check operation
        device_uuids = run_compliance_params.get("deviceUuids")
        if not device_uuids:
            self.msg = "No device UUIDs were found for the execution of the compliance operation."
            self.set_operation_result("ok", False, self.msg, "INFO")
            self.module.exit_json(**self.result)

        batches_dict = {}

        if len(device_uuids) > batch_size:
            batches = [device_uuids[i:i + batch_size] for i in range(0, len(device_uuids), batch_size)]
        else:
            batches = [device_uuids]
        self.log("Created {0} batch(es) for run compliance operation: {1}".format(len(batches), batches), "DEBUG")

        for idx, batch in enumerate(batches):
            self.log("Executing 'run_compliance' operation on batch: {0} - {1}".format(idx, batch), "DEBUG")
            batch_params = run_compliance_params.copy()
            batch_params["deviceUuids"] = batch
            self.log("Batch {0} Parameters: {1}".format(idx, batch_params), "DEBUG")

            task_id = self.get_taskid_post_api_call("compliance", "run_compliance", batch_params)
            if task_id:
                batches_dict[idx] = {"task_id": task_id, "batch_params": batch_params}
            else:
                self.log("No response received from the 'run_compliance' API call for batch: {0}.".format(batch_params), "ERROR")

        return batches_dict

    def sync_device_config(self, sync_device_config_params):
        """
        Synchronize the device configuration using the specified parameters.
        Args:
            - sync_device_config_params (dict): Parameters for synchronizing the device configuration.
        Returns:
            task_id (str): The ID of the task created for the synchronization operation.
        Note:
            This method initiates the synchronization of device configurations by making an API call to the Cisco Catalyst Center.
            It logs the response received from the API call and extracts the task ID from the response for further monitoring.
            If an error occurs during the API call, it will be caught and logged.
        """
        # Make an API call to synchronize device configuration
        return self.get_taskid_post_api_call("compliance", "commit_device_configuration", sync_device_config_params)

    def handle_error(self, task_name, mgmt_ip_to_instance_id_map, failure_reason=None):
        """
        Handle error encountered during task execution.
        Args:
            - task_name (str): Name of the task being performed.
            - mgmt_ip_to_instance_id_map (dict): Mapping of management IP addresses to instance IDs.
            - failure_reason (str, optional): Reason for the failure, if available.
        Returns:
            self (object): An instance of the class used for interacting with Cisco Catalyst Center.
        """
        # If failure reason is provided, include it in the error message
        ip_address_list_str = ", ".join(list(mgmt_ip_to_instance_id_map.keys()))
        if failure_reason:
            self.msg = "An error occurred while performing {0} on device(s): {1}. The operation failed due to the following reason: {2}".format(
                task_name, ip_address_list_str, failure_reason)
        # If no failure reason is provided, generate a generic error message
        else:
            self.msg = "An error occurred while performing {0} on device(s): {1}".format(
                task_name, ip_address_list_str)

        # Update the result with failure status and log the error message
        self.set_operation_result("failed", False, self.msg, "ERROR")

        return self

    def get_batches_result(self, batches_dict):
        """
        Retrieves the results of compliance check tasks for a list of device batches.
        Args:
            batches_dict (dict): A dictionary where each key is a batch index and the value is a dictionary
                                 containing 'task_id' and 'batch_params'.
        Returns:
            list: A list of dictionaries where each dictionary contains the 'task_id', 'batch_params',
                  'task_status', and 'msg' for each batch.
        Description:
            This function iterates over the provided batches, retrieves the task status for each batch,
            and stores the result including task ID, batch parameters, task status, and message.
        """
        batches_result = []
        task_name = "Run Compliance"

        for idx, batch_info in batches_dict.items():
            task_id = batch_info["task_id"]
            device_ids = batch_info["batch_params"]["deviceUuids"]
            success_msg = (
                "{0} Task with Task ID: '{1}' for batch number: '{2}' with {3} devices: {4} is successful."
                .format(task_name, task_id, idx, len(device_ids), device_ids)
            )

            # Get task status for the current batch
            if self.dnac_version <= self.version_2_3_5_3:
                failure_msg = (
                    "{0} Task with Task ID: '{1}' Failed for batch number: '{2}' with {3} devices: {4}."
                    .format(task_name, task_id, idx, len(device_ids), device_ids)
                )
                progress_validation = "report has been generated successfully"
                self.get_task_status_from_task_by_id(task_id, task_name, failure_msg, success_msg, progress_validation=progress_validation)
            else:
                self.get_task_status_from_tasks_by_id(task_id, task_name, success_msg)

            task_status = self.status
            self.log("The task status of batch: {0} with task id: {1} is {2}".format(idx, task_id, task_status), "INFO")

            # Store the result for the current batch
            batch_result = {
                "task_id": task_id,
                "batch_params": batch_info["batch_params"],
                "task_status": task_status,
                "msg": success_msg
            }

            # Append the current batch result to the batches_result list
            batches_result.append(batch_result)

        self.log("Collective result of all batches: {0}".format(batches_result), "DEBUG")
        return batches_result

    def validate_batch_result(self, batches_result, retried_batches=None):
        """
        Validates the results of compliance check tasks for device batches.
        Args:
            batches_status (list): A list of dictionaries where each dictionary contains 'task_id',
                                   'batch_params', 'task_status', and 'msg' for each batch.
        Returns:
            list: A list of device IDs that have successfully completed the compliance check.
        Description:
            This function iterates over the provided batches status, checks the task status for each batch,
            and collects the device IDs for batches that have completed successfully. For batches that failed,
            it re-runs the compliance check with a batch size of 1, validates the results recursively, and collects
            the successful device IDs.
        """
        if retried_batches is None:
            retried_batches = set()
        successful_devices = []

        # Iterate over each batch in the batches_status list
        for batch in batches_result:
            task_status = batch.get("task_status")
            batch_params = batch.get("batch_params")
            device_ids = tuple(batch_params.get("deviceUuids"))

            # Check if the task status is successful
            if task_status == "success":
                successful_devices.extend(device_ids)
            else:
                # Check if the batch has already been retried with batch size of 1
                if device_ids in retried_batches:
                    device_ids_str = ", ".join(device_ids)
                    self.log(
                        "Batch for device(s) {0} has already been retried with batch size of 1 and failed. "
                        "Stopping recursion.".format(device_ids_str),
                        "ERROR"
                    )
                    continue

                self.log("Re-running compliance check for batch {0} with batch_result: {1} ".format(batch, batches_result), "WARNING")
                # Re-run the compliance check for the failed batch with batch size of 1
                retried_batches.add(device_ids)
                batches_dict = self.run_compliance(batch_params, batch_size=1)
                batches_result = self.get_batches_result(batches_dict)

                # Recursively validate the batch results and append the successful device IDs
                successful_devices.extend(self.validate_batch_result(batches_result, retried_batches=retried_batches))
        msg = ("The results of all batches have been validated, and the compliance checks "
               "were successfully executed on following devices: {0}".format(successful_devices))
        self.log(msg, "DEBUG")

        return successful_devices

    def get_compliance_task_status(self, batches_dict, mgmt_ip_to_instance_id_map):
        """
        Retrieves and processes compliance check task statuses for multiple batches.
        Args:
            batches_dict (dict): A dictionary containing information about each batch of compliance tasks.
            mgmt_ip_to_instance_id_map (dict): A dictionary mapping management IP addresses to instance IDs.
        Returns:
            self
        Description:
            This function processes the compliance check statuses for multiple batches of devices. It determines
            which devices were successful and which were unsuccessful. If any batches were successful, it logs
            the success message, updates the result, and retrieves compliance reports. If all batches failed,
            it logs the failure message and updates the result accordingly.
        """
        task_name = "Run Compliance Check"
        batches_result = self.get_batches_result(batches_dict)
        successful_devices = self.validate_batch_result(batches_result)
        successful_devices_str = ", ".join(successful_devices)
        self.log("{0} successful on device(s): {1}".format(task_name, successful_devices_str), "DEBUG")

        # Reverse the mgmt_ip_to_instance_id_map to map device IDs to IPs
        id_to_ip_map = {v: k for k, v in mgmt_ip_to_instance_id_map.items()}

        # Determine unsuccessful devices
        all_device_ids = [
            device_id
            for batch in batches_dict.values()
            for device_id in batch["batch_params"]["deviceUuids"]
        ]
        unsuccessful_devices = list(set(all_device_ids) - set(successful_devices))
        unsuccessful_ips = [id_to_ip_map[device_id] for device_id in unsuccessful_devices if device_id in id_to_ip_map]

        final_msg = {}

        if successful_devices:
            successful_ips = [id_to_ip_map[device_id] for device_id in successful_devices if device_id in id_to_ip_map]
            self.log("{0} Succeeded for following device(s): {1}".format(task_name, successful_ips), "INFO")
            final_msg["{0} Succeeded for following device(s)".format(task_name)] = {
                "success_count": len(successful_ips),
                "success_devices": successful_ips
            }
            successful_devices_params = self.want.get("run_compliance_params").copy()
            successful_devices_params["deviceUuids"] = successful_devices
            compliance_report = self.get_compliance_report(successful_devices_params, mgmt_ip_to_instance_id_map)
            self.log("Compliance Report for device on which compliance operation was successful: {0}".format(compliance_report), "INFO")

        if unsuccessful_ips:
            self.log("{0} Failed for following device(s): {1}".format(task_name, unsuccessful_ips), "ERROR")
            final_msg["{0} Failed for following device(s)".format(task_name)] = {
                "failed_count": len(unsuccessful_ips),
                "failed_devices": unsuccessful_ips
            }

        self.msg = final_msg

        # Determine the final operation result
        if successful_devices and unsuccessful_devices:
            self.set_operation_result("failed", True, self.msg, "ERROR", compliance_report)
        elif successful_devices:
            self.set_operation_result("success", True, self.msg, "INFO", compliance_report)
        elif unsuccessful_devices:
            self.set_operation_result("failed", True, self.msg, "ERROR")
        else:
            self.set_operation_result("ok", False, self.msg, "INFO")

        return self

    def get_sync_config_task_status(self, task_id, mgmt_ip_to_instance_id_map):
        """
        This function manages the status of device configuration synchronization tasks in Cisco Catalyst Center.
        Args:
            - task_id: ID of the synchronization task
            - mgmt_ip_to_instance_id_map: Mapping of management IP addresses to instance IDs
        Returns:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
        Description:
            It validates if synchronization is required, categorizes devices based on compliance status, and checks task completion status.
            If all devices are already compliant, it logs a success message. If some devices have unexpected statuses, it logs an error.
            It continuously checks the task status until completion, updating the result accordingly.
        """
        task_name = "Sync Device Configuration"
        self.log("Entering '{0}' with task_id: '{1}' and mgmt_ip_to_instance_id_map: {2}".format(
            task_name, task_id, mgmt_ip_to_instance_id_map), "INFO"
        )
        msg = {}

        # Retrieve the parameters for sync device config
        sync_device_config_params = self.want.get("sync_device_config_params")
        self.log("Sync device config parameters: {0}".format(sync_device_config_params), "DEBUG")

        # Extract the list of device IDs from sync_device_config_params
        device_ids = sync_device_config_params.get("deviceId")
        self.log("Device IDs for synchronization: {0}".format(device_ids), "DEBUG")

        # Create device_ip_list by mapping the device IDs back to their corresponding IP addresses
        device_ip_list = [ip for ip, device_id in mgmt_ip_to_instance_id_map.items() if device_id in device_ids]
        self.log("Device IPs to synchronize: {0}".format(device_ip_list), "DEBUG")

        msg["{0} Succeeded for following device(s)".format(task_name)] = {"success_count": len(device_ip_list), "success_devices": device_ip_list}

        # Retrieve and return the task status using the provided task ID
        return self.get_task_status_from_tasks_by_id(task_id, task_name, msg)

    def process_final_result(self, final_status_list):
        """
        Processes a list of final statuses and returns a tuple indicating the result and a boolean flag.
        Args:
            final_status_list (list): List of status strings to process.
        Returns:
            tuple: A tuple containing a status string ("ok" or "success") and a boolean flag (False if all statuses are "ok", True otherwise).
        """
        status_set = set(final_status_list)

        if status_set == {"ok"}:
            return "ok", False
        else:
            return "success", True

    def verify_sync_device_config(self):
        """
        Verify the success of the "Sync Device Configuration" operation.
        Args:
            config (dict): A dictionary containing the configuration details.
        Returns:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
        Description:
            This method verifies the success of the "Sync Device Configuration" operation in the context of network compliance management.
            It checks if the configuration includes the option to synchronize device configurations (`sync_device_config`).
            If this option is present, the function proceeds to compare compliance details before and after executing the synchronization operation.
            It logs relevant information at each step and concludes by determining whether the synchronization was successful.
        """
        # Get compliance details before running sync_device_config
        compliance_details_before = self.want.get("compliance_details")
        self.log("Compliance details before running sync_device_config: {0}".format(compliance_details_before), "INFO")

        # Get compliance details after running sync_device_config
        compliance_details_after = self.get_compliance_report(
            self.want.get("compliance_detail_params_sync"),
            self.want.get("mgmt_ip_to_instance_id_map")
        )
        if not compliance_details_after:
            self.msg = "Error occurred when Retrieving Compliance Details after for verifying configuration."
            self.fail_and_exit(self.msg)

        self.log("Compliance details after running sync_device_config: {0}.".format(compliance_details_after), "INFO")

        # Get the device IDs to check
        sync_device_ids = self.want.get("sync_device_config_params").get("deviceId", [])
        if not sync_device_ids:
            self.log(
                "No device IDs found in sync_device_config_params, Sync Device Configuration "
                "operation may not have been performed.",
                "WARNING"
            )
            return self

        # Initialize the status lists
        all_statuses_before = []
        all_statuses_after = []

        # Iterate over the device IDs and check their compliance status
        self.log("Device IDs to check: {0}".format(sync_device_ids), "DEBUG")
        for device_id in sync_device_ids:
            # Find the corresponding IP address from the mgmt_ip_to_instance_id_map
            ip_address = next((ip for ip, id in self.want.get("mgmt_ip_to_instance_id_map").items() if id == device_id), None)
            self.log("Found IP address for device ID {0}: {1}".format(device_id, ip_address), "DEBUG")
            if ip_address:
                self.log("Checking compliance status for device ID: {0}".format(device_id), "DEBUG")
                # Get the status before
                compliance_before = compliance_details_before.get(ip_address, [])
                if compliance_before:
                    all_statuses_before.append(compliance_before[0]["status"])
                else:
                    self.log("No compliance details found for device IP: {0} before synchronization.".format(ip_address), "DEBUG")
                # Get the status after
                compliance_after = compliance_details_after.get(ip_address, [])
                if compliance_after:
                    all_statuses_after.append(compliance_after[0]["status"])
                else:
                    self.log("No compliance details found for device IP: {0} after synchronization.".format(ip_address), "DEBUG")

                self.log("Compliance statuses before synchronization: {0}".format(all_statuses_before), "DEBUG")
                self.log("Compliance statuses after synchronization: {0}".format(all_statuses_after), "DEBUG")

            else:
                self.log("No IP address found for device ID: {0}".format(device_id), "DEBUG")

        # Check if all statuses changed from "NON_COMPLIANT" to "COMPLIANT"
        if (
            all(all_status == "NON_COMPLIANT" for all_status in all_statuses_before) and
            all(all_status == "COMPLIANT" for all_status in all_statuses_after)
        ):
            self.log("Verified the success of the Sync Device Configuration operation.")
        else:
            self.log(
                "Sync Device Configuration operation may have been unsuccessful "
                "since not all devices have 'COMPLIANT' status after the operation.",
                "WARNING"
            )

    def get_want(self, config):
        """
        Determines the desired state based on the provided configuration.
        Args:
            config (dict): The configuration specifying the desired state.
        Returns:
            dict: A dictionary containing the desired state parameters.
        Description:
            This method processes the provided configuration to determine the desired state. It validates the presence of
            either "ip_address_list" or "site_name" and constructs parameters for running compliance checks and syncing
            device configurations based on the provided configuration. It also logs the desired state for reference.
        """
        # Initialize parameters
        run_compliance_params = {}
        sync_device_config_params = {}
        compliance_detail_params_sync = {}
        compliance_details = {}

        # Store input parameters
        ip_address_list = config.get("ip_address_list")
        self.log("Original IP address list: {0}".format(ip_address_list), "DEBUG")
        # Remove Duplicates from list
        if ip_address_list:
            ip_address_list = list(set(ip_address_list))
            self.log("Deduplicated IP address list: {0}".format(ip_address_list), "DEBUG")
        site_name = config.get("site_name")

        run_compliance = config.get("run_compliance")
        run_compliance_categories = config.get("run_compliance_categories")
        sync_device_config = config.get("sync_device_config")

        # Validate the provided configuration parameters
        self.validate_params(config)

        # Retrieve device ID list
        mgmt_ip_to_instance_id_map = self.get_device_id_list(ip_address_list, site_name)
        self.log("Management IP to Instance ID Map: {0}".format(mgmt_ip_to_instance_id_map), "DEBUG")

        # Run Compliance Paramters
        run_compliance_params = self.get_run_compliance_params(mgmt_ip_to_instance_id_map, run_compliance, run_compliance_categories)

        # Sync Device Configuration Parameters
        if sync_device_config:
            self.log("Sync Device Configuration is requested.", "DEBUG")
            if self.dnac_version > self.version_2_3_5_3:
                compliance_detail_params_sync = {
                    "deviceUuids": list(mgmt_ip_to_instance_id_map.values()),
                    "categories": ["RUNNING_CONFIG"]
                }
                self.log("Retrieving compliance report with parameters: {0}".format(compliance_detail_params_sync), "DEBUG")
                response = self.get_compliance_report(compliance_detail_params_sync, mgmt_ip_to_instance_id_map)
                self.log("Response from get_compliance_report: {0}".format(response), "DEBUG")
                if not response:
                    ip_address_list_str = ", ".join(list(mgmt_ip_to_instance_id_map.keys()))
                    self.msg = "Error occurred when retrieving Compliance Report to identify if Sync Device Config Operation "
                    self.msg += "is required on device(s): {0}".format(ip_address_list_str)
                    self.fail_and_exit(self.msg)

                compliance_details = response
                sync_required, self.msg, categorized_devices = self.is_sync_required(response, mgmt_ip_to_instance_id_map)
                self.log("Is Sync Requied: {0} -- Message: {1}".format(sync_required, self.msg), "DEBUG")
                if sync_required:
                    sync_device_config_params = self.get_sync_device_config_params(mgmt_ip_to_instance_id_map, categorized_devices)
                    self.log(
                        "Sync Device Configuration operation is required for provided parameters in the Cisco Catalyst Center."
                        "therefore setting 'sync_device_config_params' - {0}.".format(sync_device_config_params),
                        "DEBUG"
                    )
                else:
                    self.log(
                        "Sync Device Configuration operation is not required for provided parameters in the Cisco Catalyst Center."
                        "therefore not setting the 'sync_device_config_params'",
                        "INFO"
                    )
                    self.skipped_sync_device_configs_list.extend(list(mgmt_ip_to_instance_id_map.keys()))
            else:
                self.msg = (
                    "The specified Cisco Catalyst Center version: '{0}' does not support the Sync Device Config operation. "
                    "Supported version start '2.3.7.6' onwards. Version '2.3.5.3' introduces APIs for "
                    "performing Compliance Checks. Version '2.3.7.6' expands support to include APIs "
                    "for Compliance Checks and Sync Device Config operations.".format(self.get_ccc_version())
                )
                self.fail_and_exit(self.msg)

        # Construct the "want" dictionary containing the desired state parameters
        want = {}
        want = dict(
            ip_address_list=ip_address_list,
            site_name=site_name,
            mgmt_ip_to_instance_id_map=mgmt_ip_to_instance_id_map,
            run_compliance_params=run_compliance_params,
            run_compliance_batch_size=config.get("run_compliance_batch_size"),
            sync_device_config_params=sync_device_config_params,
            compliance_detail_params_sync=compliance_detail_params_sync,
            compliance_details=compliance_details
        )
        self.want = want
        self.log("Desired State (want): {0}".format(str(self.want)), "INFO")

        return self

    def get_diff_merged(self, config):
        """
        This method is designed to Perform Network Compliance Actions in Cisco Catalyst Center.
        Args: None
        Returns:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
        Description:
            This method orchestrates compliance check operation and device configuration synchronization tasks specified in a playbook.
            It ensures all required tasks are present, executes them, and checks their status, facilitating smooth playbook execution.
        """
        # Action map for different network compliance operations
        self.log("Starting 'get_diff_merged' operation.", "INFO")

        action_map = {
            "run_compliance_params": (self.run_compliance, self.get_compliance_task_status),
            "sync_device_config_params": (self.sync_device_config, self.get_sync_config_task_status)
        }

        # Check if all action_map keys are missing in self.want
        if not any(action_param in self.want for action_param in action_map.keys()):
            self.msg = "Network Compliance operation(s) are not required for the provided input parameters in the Cisco Catalyst Center."
            self.set_operation_result("ok", False, self.msg, "INFO")
            return self

        final_status_list = []
        result_details = {}

        # Iterate through the action map and execute specified actions
        for action_param, (action_func, status_func) in action_map.items():
            req_action_param = self.want.get(action_param)
            # Execute the action and check its status
            if req_action_param:
                self.log("Executing action function: {0} with params: {1}".format(action_func.__name__, req_action_param), "INFO")
                if action_param == "run_compliance_params":
                    batch_size = self.want.get("run_compliance_batch_size")
                    result_task_id = action_func(self.want.get(action_param), batch_size=batch_size)
                else:
                    result_task_id = action_func(self.want.get(action_param))

                if not result_task_id:
                    self.msg = "An error occurred while retrieving the task_id of the {0} operation.".format(action_func.__name__)
                    self.set_operation_result("failed", False, self.msg, "CRITICAL")
                else:
                    self.log("Task Id: {0} returned from the action function: {1}".format(result_task_id, action_func.__name__), "DEBUG")
                    status_func(result_task_id, self.want.get("mgmt_ip_to_instance_id_map")).check_return_status()
                    result = self.msg
                    result_details.update(result)

        if config.get("sync_device_config"):
            skipped_sync_device_configs_list = set(self.skipped_sync_device_configs_list)
            if skipped_sync_device_configs_list:
                self.log("Sync Device Configuration skipped for devices: {0}".format(skipped_sync_device_configs_list), "DEBUG")
                result_details["Sync Device Configuration operation Skipped for following device(s)"] = {
                    "skipped_count": len(skipped_sync_device_configs_list),
                    "skipped_devices": skipped_sync_device_configs_list
                }

        skipped_run_compliance_devices_list = set(self.skipped_run_compliance_devices_list)
        if skipped_run_compliance_devices_list:
            self.log("Run Compliance Check skipped for devices: {0}".format(skipped_run_compliance_devices_list), "DEBUG")
            result_details["Run Compliance Check Skipped for following device(s)"] = {
                "skipped_count": len(skipped_run_compliance_devices_list),
                "skipped_devices": skipped_run_compliance_devices_list
            }

        final_status, is_changed = self.process_final_result(final_status_list)
        self.msg = result_details
        self.log("Completed 'get_diff_merged' operation with final status: {0}, is_changed: {1}".format(final_status, is_changed), "INFO")
        self.set_operation_result(final_status, is_changed, self.msg, "INFO", self.result.get("data"))
        return self

    def verify_diff_merged(self, config):
        """
        Verify the success of the "Sync Device Configuration" operation.
        Args:
            config (dict): A dictionary containing the configuration details.
        Returns:
            self (object): An instance of a class used for interacting with Cisco Catalyst Center.
        Description:
            This method verifies the success of the "Sync Device Configuration" operation in the context of network compliance management.
            It checks if the configuration includes the option to synchronize device configurations (`sync_device_config`).
            If this option is present, the function proceeds to compare compliance details before and after executing the synchronization operation.
            It logs relevant information at each step and concludes by determining whether the synchronization was successful.
        """
        self.log("Starting 'verify_diff_merged' operation.", "INFO")

        sync_device_config_params = self.want.get("sync_device_config_params")
        run_compliance_params = self.want.get("run_compliance_params")

        if sync_device_config_params:
            self.log("Starting verification of Sync Device Configuration operation.", "INFO")
            self.verify_sync_device_config()
            self.log("Completed verification of Sync Device Configuration operation.", "INFO")

        if run_compliance_params:
            self.log("Verification of configuration is not required for Run Compliance operation!", "INFO")

        self.log("Completed 'verify_diff_merged' operation.", "INFO")
        return self


def main():
    """
    main entry point for module execution
    """

    # Define the specification for the module"s arguments
    element_spec = {"dnac_host": {"required": True, "type": "str"},
                    "dnac_port": {"type": "str", "default": "443"},
                    "dnac_username": {"type": "str", "default": "admin", "aliases": ["user"]},
                    "dnac_password": {"type": "str", "no_log": True},
                    "dnac_verify": {"type": "bool", "default": "True"},
                    "dnac_version": {"type": "str", "default": "2.2.3.3"},
                    "dnac_debug": {"type": "bool", "default": False},
                    "dnac_log_level": {"type": "str", "default": "WARNING"},
                    "dnac_log_file_path": {"type": "str", "default": "dnac.log"},
                    "dnac_log_append": {"type": "bool", "default": True},
                    "dnac_log": {"type": "bool", "default": False},
                    "validate_response_schema": {"type": "bool", "default": True},
                    "config_verify": {"type": "bool", "default": False},
                    "dnac_api_task_timeout": {"type": "int", "default": 1200},
                    "dnac_task_poll_interval": {"type": "int", "default": 2},
                    "config": {"required": True, "type": "list", "elements": "dict"},
                    "state": {"default": "merged", "choices": ["merged"]}
                    }

    # Initialize the Ansible module with the provided argument specifications
    module = AnsibleModule(argument_spec=element_spec,
                           supports_check_mode=False)

    # Initialize the NetworkCompliance object with the module
    ccc_network_compliance = NetworkCompliance(module)

    # Get the state parameter from the provided parameters
    state = ccc_network_compliance.params.get("state")

    # Check if the state is valid
    if state not in ccc_network_compliance.supported_states:
        ccc_network_compliance.status = "invalid"
        ccc_network_compliance.msg = "State {0} is invalid".format(state)
        ccc_network_compliance.check_return_status()

    # Validate the input parameters and check the return status
    ccc_network_compliance.validate_input().check_return_status()

    # Get the config_verify parameter from the provided parameters
    config_verify = ccc_network_compliance.params.get("config_verify")

    # Iterate over the validated configuration parameters
    for config in ccc_network_compliance.validated_config:
        ccc_network_compliance.get_want(config).check_return_status()
        ccc_network_compliance.get_diff_state_apply[state](config).check_return_status()
        if config_verify:
            ccc_network_compliance.verify_diff_state_apply[state](config).check_return_status()

    # Exit with the result obtained from the NetworkCompliance object
    module.exit_json(**ccc_network_compliance.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_layer2_virtual_networks_v1_info
short_description: Information module for Sda Layer2 Virtual Networks V1
description:
- Get all Sda Layer2 Virtual Networks V1.
- Returns a list of layer 2 virtual networks that match the provided query parameters.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id query parameter. ID of the layer 2 virtual network.
    type: str
  fabricId:
    description:
    - FabricId query parameter. ID of the fabric the layer 2 virtual network is assigned to.
    type: str
  vlanName:
    description:
    - VlanName query parameter. The vlan name of the layer 2 virtual network.
    type: str
  vlanId:
    description:
    - VlanId query parameter. The vlan ID of the layer 2 virtual network.
    type: float
  trafficType:
    description:
    - TrafficType query parameter. The traffic type of the layer 2 virtual network.
    type: str
  associatedLayer3VirtualNetworkName:
    description:
    - AssociatedLayer3VirtualNetworkName query parameter. Name of the associated layer 3 virtual network.
    type: str
  offset:
    description:
    - Offset query parameter. Starting record for pagination.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA GetLayer2VirtualNetworksV1
  description: Complete reference of the GetLayer2VirtualNetworksV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-layer-2-virtual-networks
notes:
  - SDK Method used are
    sda.Sda.get_layer2_virtual_networks_v1,

  - Paths used are
    get /dna/intent/api/v1/sda/layer2VirtualNetworks,

"""

EXAMPLES = r"""
- name: Get all Sda Layer2 Virtual Networks V1
  cisco.dnac.sda_layer2_virtual_networks_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
    fabricId: string
    vlanName: string
    vlanId: 0
    trafficType: string
    associatedLayer3VirtualNetworkName: string
    offset: 0
    limit: 0
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "id": "string",
          "fabricId": "string",
          "vlanName": "string",
          "vlanId": 0,
          "trafficType": "string",
          "isFabricEnabledWireless": true,
          "associatedLayer3VirtualNetworkName": "string"
        }
      ],
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2018 Fred-sun, <xiuxi.sun@qq.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_datafactory_info
version_added: "0.1.12"
short_description: Get data factory facts
description:
    - Get facts for a specific data factory.

options:
    name:
        description:
            - The factory name.
        type: str
    resource_group:
        description:
            - Limit results by resource group. Required when using name parameter.
        type: str
    if_none_match:
        description:
            - ETag of the factory entity.
            - Should only be specified for get.
            - If the ETag matches the existing entity tag, or if * was provided, then no content will be returned.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Fred-sun (@Fred-sun)
    - xuzhang3 (@xuzhang3)
'''

EXAMPLES = '''
- name: Get data factory by name
  azure_rm_datafactory_info:
    resource_group: "{{ resource_group }}"
    name: "{{ name }}"

- name: Get data factory by resource group
  azure_rm_datafactory_info:
    resource_group: "{{ resource_group }}"

- name: Get data factory in relate subscription
  azure_rm_datafactory_info:
    tags:
      - key1
'''

RETURN = '''
datafactory:
    description:
        - Current state fo the data factory.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The data facotry ID.
            type: str
            returned: always
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DataFactory/factories/testpro"
        create_time:
            description:
                - Time the factory was created in ISO8601 format.
            type: str
            returned: always
            sample: "2022-04-26T08:24:41.391164+00:00"
        location:
            description:
                - The resource location.
            type: str
            returned: always
            sample: eastus
        name:
            description:
                - The resource name.
            type: str
            returned: always
            sample: testfactory
        provisioning_state:
            description:
                - Factory provisioning state, example Succeeded.
            type: str
            returned: always
            sample: Succeeded
        e_tag:
            description:
                - Etag identifies change in the resource.
            type: str
            returned: always
            sample: "3000fa80-0000-0100-0000-6267ac490000"
        type:
            description:
                - The resource type.
            type: str
            returned: always
            sample: "Microsoft.DataFactory/factories"
        public_network_access:
            description:
                - Whether or not public network access is allowed for the data factory.
            type: str
            returned: always
            sample: "Enabled"
        tags:
            description:
                - List the data factory tags.
            type: str
            returned: always
            sample: {'key1':'value1'}
        identity:
            description:
                -  Managed service identity of the factory.
            type: str
            returned: always
            contains:
                principal_id:
                    description:
                        - The principal id of the identity.
                    type: str
                    returned: always
                    sample: "***********"
                tenant_id:
                    description:
                        - The client tenant id of the identity.
                    type: str
                    returned: always
                    sample: "***********"
        repo_configuration:
            description:
                - Git repo information of the factory.
            type: str
            returned: always
            contains:
                type:
                    description:
                        - Type of repo configuration.
                    type: str
                    returned: always
                    sample: FactoryGitHubConfiguration
                ccount_name:
                    description:
                        - Account name.
                    type: str
                    returned: always
                    sample: fredaccount
                collaboration_branch:
                    description:
                        - Collaboration branch.
                    type: str
                    returned: always
                    sample: branch
                repository_name:
                    description:
                        - Repository name.
                    type: str
                    returned: always
                    sample:  "vault"
                root_folder:
                    description:
                        - Root folder.
                    type: str
                    returned: always
                    sample: "/home/"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

AZURE_OBJECT_CLASS = 'DataFactoryInfo'


class AzureRMDataFactoryInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            if_none_match=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None
        self.if_none_match = None
        self.tags = None

        super(AzureRMDataFactoryInfo, self).__init__(self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     supports_tags=False,
                                                     facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        result = []

        if self.name and self.resource_group:
            result = self.get_item()
        elif self.resource_group:
            result = self.list_by_resourcegroup()
        else:
            result = self.list_all()

        self.results['datafactory'] = self.format(result)

        return self.results

    def format(self, raw):
        results = []
        for item in raw:
            if self.has_tags(item.tags, self.tags):
                results.append(self.pip_to_dict(item))
        return results

    def pip_to_dict(self, pip):
        result = dict(
            id=pip.id,
            name=pip.name,
            type=pip.type,
            location=pip.location,
            tags=pip.tags,
            e_tag=pip.e_tag,
            provisioning_state=pip.provisioning_state,
            create_time=pip.create_time,
            repo_configuration=dict(),
            identity=dict(),
            public_network_access=pip.public_network_access,
        )
        if pip.identity:
            result['identity']['principal_id'] = pip.identity.principal_id
            result['identity']['tenant_id'] = pip.identity.tenant_id
        if pip.repo_configuration:
            result['repo_configuration']['type'] = pip.repo_configuration.type
            result['repo_configuration']['account_name'] = pip.repo_configuration.account_name
            result['repo_configuration']['repository_name'] = pip.repo_configuration.repository_name
            result['repo_configuration']['collaboration_branch'] = pip.repo_configuration.collaboration_branch
            result['repo_configuration']['root_folder'] = pip.repo_configuration.root_folder
            if pip.repo_configuration.type == "FactoryVSTSConfiguration":
                result['repo_configuration']['project_name'] = pip.repo_configuration.project_name
        return result

    def get_item(self):
        response = None
        self.log('Get properties for {0}'.format(self.name))
        try:
            response = self.datafactory_client.factories.get(self.resource_group, self.name, self.if_none_match)
        except ResourceNotFoundError:
            pass
        return [response] if response else []

    def list_by_resourcegroup(self):
        self.log("Get GitHub Access Token Response")
        try:
            response = self.datafactory_client.factories.list_by_resource_group(self.resource_group)
        except Exception:
            pass
        return response if response else []

    def list_all(self):
        self.log("Get GitHub Access Token Response")
        try:
            response = self.datafactory_client.factories.list()
        except Exception:
            pass
        return response if response else []


def main():
    AzureRMDataFactoryInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: authentication_import_certificate_v1
short_description: Resource module for Authentication Import Certificate V1
description:
- Manage operation create of the resource Authentication Import Certificate V1.
- This API enables a user to import a PEM certificate and its key for the controller and/or disaster recovery.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  listOfUsers:
    description: ListOfUsers query parameter. Specify whether the certificate will be
      used for controller ("server"), disaster recovery ("ipsec") or both ("server,
      ipsec"). If no value is provided, the default value taken will be "server".
    elements: dict
    suboptions:
      description:
        description: Authentication Import Certificate's listOfUsers.
        type: str
    type: list
  pkPassword:
    description: PkPassword query parameter. Password for encrypted private key.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Authentication Management ImportCertificateV1
  description: Complete reference of the ImportCertificateV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!import-certificate
notes:
  - SDK Method used are
    authentication_management.AuthenticationManagement.import_certificate_v1,

  - Paths used are
    post /dna/intent/api/v1/certificate,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.authentication_import_certificate_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    listOfUsers: []
    pkPassword: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2019 Hai Cao, <t-haicao@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_cdnendpoint_info

version_added: "0.1.2"

short_description: Get Azure CDN endpoint facts

description:
    - Get facts for a specific Azure CDN endpoint or all Azure CDN endpoints.

options:
    resource_group:
        description:
            - Name of resource group where this CDN profile belongs to.
        required: true
        type: str
    profile_name:
        description:
            - Name of CDN profile.
        required: true
        type: str
    name:
        description:
            - Limit results to a specific Azure CDN endpoint.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Hai Cao (@caohai)
    - Yunge zhu (@yungezz)
'''

EXAMPLES = '''
- name: Get facts for all endpoints in CDN profile
  azure_rm_cdnendpoint_info:
    resource_group: myResourceGroup
    profile_name: myCDNProfile
    tags:
      - key
      - key:value

- name: Get facts of specific CDN endpoint
  azure_rm_cdnendpoint_info:
    resource_group: myResourceGroup
    profile_name: myCDNProfile
    name: myEndpoint1
'''

RETURN = '''
cdnendpoints:
    description: List of Azure CDN endpoints.
    returned: always
    type: complex
    contains:
        resource_group:
            description:
                - Name of a resource group where the Azure CDN endpoint exists.
            returned: always
            type: str
            sample: myResourceGroup
        name:
            description:
                - Name of the Azure CDN endpoint.
            returned: always
            type: str
            sample: myEndpoint
        profile_name:
            description:
                - Name of the Azure CDN profile that this endpoint is attached to.
            returned: always
            type: str
            sample: myProfile
        location:
            description:
                - Location of the Azure CDN endpoint.
            type: str
            sample: WestUS
        id:
            description:
                - ID of the Azure CDN endpoint.
            type: str
            sample:
                "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myCDN/providers/Microsoft.Cdn/profiles/myProfile/endpoints/myEndpoint1"
        provisioning_state:
            description:
                - Provisioning status of the Azure CDN endpoint.
            type: str
            sample: Succeeded
        resource_state:
            description:
                - Resource status of the profile.
            type: str
            sample: Running
        is_compression_enabled:
            description:
                - Indicates whether content compression is enabled on CDN.
            type: bool
            sample: true
        is_http_allowed:
            description:
                - Indicates whether HTTP traffic is allowed on the endpoint.
            type: bool
            sample: true
        is_https_allowed:
            description:
                - Indicates whether HTTPS traffic is allowed on the endpoint.
            type: bool
            sample: true
        query_string_caching_behavior:
            description:
                - Defines how CDN caches requests that include query strings.
            type: str
            sample: IgnoreQueryString
        content_types_to_compress:
            description:
                - List of content types on which compression applies.
            type: list
            sample: [
                "text/plain",
                "text/html",
                "text/css",
                "text/javascript",
                "application/x-javascript",
                "application/javascript",
                "application/json",
                "application/xml"
            ]
        origins:
            description:
                - The source of the content being delivered via CDN.
            sample: {
                "host_name": "xxxxxxxx.blob.core.windows.net",
                "http_port": null,
                "https_port": null,
                "name": "xxxxxxxx-blob-core-windows-net"
            }
            type: dict
        origin_host_header:
            description:
                - The host header value sent to the origin with each request.
            type: str
            sample: xxxxxxxx.blob.core.windows.net
        origin_path:
            description:
                - A directory path on the origin that CDN can use to retrieve content from.
            type: str
            sample: /pic/
        tags:
            description:
                - The tags of the Azure CDN endpoint.
            type: list
            sample: foo
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.cdn import CdnManagementClient
except ImportError:
    # handled in azure_rm_common
    pass

import re

AZURE_OBJECT_CLASS = 'endpoints'


class AzureRMCdnEndpointInfo(AzureRMModuleBase):
    """Utility class to get Azure Azure CDN endpoint facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(type='str'),
            resource_group=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )

        self.results = dict(
            changed=False,
            cdnendpoints=[]
        )

        self.name = None
        self.resource_group = None
        self.profile_name = None
        self.tags = None

        super(AzureRMCdnEndpointInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_cdnendpoint_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_cdnendpoint_facts' module has been renamed to 'azure_rm_cdnendpoint_info'", version=(2.9, ))

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        self.cdn_client = self.get_mgmt_svc_client(CdnManagementClient,
                                                   base_url=self._cloud_environment.endpoints.resource_manager,
                                                   api_version='2017-04-02')

        if self.name:
            self.results['cdnendpoints'] = self.get_item()
        else:
            self.results['cdnendpoints'] = self.list_by_profile()

        return self.results

    def get_item(self):
        """Get a single Azure Azure CDN endpoint"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.cdn_client.endpoints.get(
                self.resource_group, self.profile_name, self.name)
        except Exception:
            pass

        if item and self.has_tags(item.tags, self.tags):
            result = [self.serialize_cdnendpoint(item)]

        return result

    def list_by_profile(self):
        """Get all Azure Azure CDN endpoints within an Azure CDN profile"""

        self.log('List all Azure CDN endpoints within an Azure CDN profile')

        try:
            response = self.cdn_client.endpoints.list_by_profile(
                self.resource_group, self.profile_name)
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(self.serialize_cdnendpoint(item))

        return results

    def serialize_cdnendpoint(self, cdnendpoint):
        '''
        Convert a Azure CDN endpoint object to dict.
        :param cdn: Azure CDN endpoint object
        :return: dict
        '''
        result = self.serialize_obj(cdnendpoint, AZURE_OBJECT_CLASS)

        new_result = {}
        new_result['id'] = cdnendpoint.id
        new_result['resource_group'] = re.sub('\\/.*', '', re.sub('.*resourcegroups\\/', '', result['id']))
        new_result['profile_name'] = re.sub('\\/.*', '', re.sub('.*profiles\\/', '', result['id']))
        new_result['name'] = cdnendpoint.name
        new_result['type'] = cdnendpoint.type
        new_result['location'] = cdnendpoint.location
        new_result['resource_state'] = cdnendpoint.resource_state
        new_result['provisioning_state'] = cdnendpoint.provisioning_state
        new_result['query_string_caching_behavior'] = cdnendpoint.query_string_caching_behavior
        new_result['is_compression_enabled'] = cdnendpoint.is_compression_enabled
        new_result['is_http_allowed'] = cdnendpoint.is_http_allowed
        new_result['is_https_allowed'] = cdnendpoint.is_https_allowed
        new_result['content_types_to_compress'] = cdnendpoint.content_types_to_compress
        new_result['origin_host_header'] = cdnendpoint.origin_host_header
        new_result['origin_path'] = cdnendpoint.origin_path
        new_result['origin'] = dict(
            name=cdnendpoint.origins[0].name,
            host_name=cdnendpoint.origins[0].host_name,
            http_port=cdnendpoint.origins[0].http_port,
            https_port=cdnendpoint.origins[0].https_port
        )
        new_result['tags'] = cdnendpoint.tags
        return new_result


def main():
    """Main module execution code path"""

    AzureRMCdnEndpointInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_analytics_endpoints_v1
short_description: Resource module for Endpoint Analytics Endpoints V1
description:
- Manage operations create, update and delete of the resource Endpoint Analytics Endpoints V1.
- Register a new endpoint in the system.
- Deletes the endpoint for the given unique identifier 'epId'.
- Update attributes of a registered endpoint.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceType:
    description: Type of the device represented by this endpoint.
    type: str
  epId:
    description: EpId path parameter. Unique identifier for the endpoint.
    type: str
  hardwareManufacturer:
    description: Hardware manufacturer for the endpoint.
    type: str
  hardwareModel:
    description: Hardware model of the endpoint.
    type: str
  macAddress:
    description: MAC address of the endpoint.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for AI Endpoint Analytics RegisterAnEndpointV1
  description: Complete reference of the RegisterAnEndpointV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!register-an-endpoint
- name: Cisco DNA Center documentation for AI Endpoint Analytics DeleteAnEndpointV1
  description: Complete reference of the DeleteAnEndpointV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-an-endpoint
- name: Cisco DNA Center documentation for AI Endpoint Analytics UpdateARegisteredEndpointV1
  description: Complete reference of the UpdateARegisteredEndpointV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-a-registered-endpoint
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.delete_an_endpoint_v1,
    aiendpoint_analytics.AIEndpointAnalytics.register_an_endpoint_v1,
    aiendpoint_analytics.AIEndpointAnalytics.update_a_registered_endpoint_v1,

  - Paths used are
    post /dna/intent/api/v1/endpoint-analytics/endpoints,
    delete /dna/intent/api/v1/endpoint-analytics/endpoints/{epId},
    put /dna/intent/api/v1/endpoint-analytics/endpoints/{epId},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.endpoint_analytics_endpoints_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    deviceType: string
    hardwareManufacturer: string
    hardwareModel: string
    macAddress: string

- name: Update by id
  cisco.dnac.endpoint_analytics_endpoints_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    deviceType: string
    epId: string
    hardwareManufacturer: string
    hardwareModel: string

- name: Delete by id
  cisco.dnac.endpoint_analytics_endpoints_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    epId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_lsm_gateway
short_description: Manages lsm-gateway objects on Checkpoint over Web Services API
description:
  - Manages lsm-gateway objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "2.3.0"
author: "Shiran Golzar (@chkp-shirango)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  security_profile:
    description:
      - LSM profile.
    type: str
  provisioning_settings:
    description:
      - Provisioning settings.
    type: dict
    suboptions:
      provisioning_profile:
        description:
          - Provisioning profile.
        type: str
  provisioning_state:
    description:
      - Provisioning state. By default the state is 'manual'- enable provisioning but not attach to profile.
      - If 'using-profile' state is provided a provisioning profile must be provided in provisioning-settings.
    type: str
    choices: ['off', 'manual', 'using-profile']
  sic:
    description:
      - Secure Internal Communication.
    type: dict
    suboptions:
      ip_address:
        description:
          - IP address. When IP address is provided- initiate trusted communication immediately using this IP address.
        type: str
      one_time_password:
        description:
          - One-time password. When one-time password is provided without ip-address- trusted communication is automatically initiated  when the
            gateway connects to the Security Management server for the first time.
        type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-lsm-gateway
  cp_mgmt_lsm_gateway:
    name: lsm_gateway
    provisioning_settings:
      provisioning_profile: prv_profile
    provisioning_state: using-profile
    security_profile: lsm_profile
    sic:
      ip_address: 1.2.3.4
      one_time_password: aaaa
    state: present

- name: set-lsm-gateway
  cp_mgmt_lsm_gateway:
    name: lsm_gateway
    provisioning_settings:
      provisioning_profile: prv_profile
    provisioning_state: using-profile
    security_profile: lsm_profile
    sic:
      ip_address: 1.2.3.4
      one_time_password: aaaa
    state: present

- name: delete-lsm-gateway
  cp_mgmt_lsm_gateway:
    name: lsm_gateway
    state: absent
"""

RETURN = """
cp_mgmt_lsm_gateway:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        security_profile=dict(type="str"),
        provisioning_settings=dict(
            type="dict", options=dict(provisioning_profile=dict(type="str"))
        ),
        provisioning_state=dict(
            type="str", choices=["off", "manual", "using-profile"]
        ),
        sic=dict(
            type="dict",
            options=dict(
                ip_address=dict(type="str"),
                one_time_password=dict(type="str", no_log=True),
            ),
        ),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "lsm-gateway"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lan_automation_v2
short_description: Resource module for Lan Automation V2
description:
- Manage operation create of the resource Lan Automation V2.
- >
   Invoke V2 LAN Automation Start API, which supports optional auto-stop processing feature based on the provided
   timeout or a specific device list, or both. The stop processing will be executed automatically when either of the
   cases is satisfied, without specifically calling the stop API. The V2 API behaves similarly to V1 if no timeout or
   device list is provided, and the user needs to call the stop API for LAN Automation stop processing. With the V2
   API, the user can also specify the level up to which the devices can be LAN automated.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Lan Automation V2's payload.
    elements: dict
    suboptions:
      discoveredDeviceSiteNameHierarchy:
        description: Discovered device site name.
        type: str
      discoveryDevices:
        description: Lan Automation V2's discoveryDevices.
        elements: dict
        suboptions:
          deviceHostName:
            description: Hostname of the device.
            type: str
          deviceManagementIPAddress:
            description: Management IP Address of the device.
            type: str
          deviceSerialNumber:
            description: Serial number of the device.
            type: str
          deviceSiteNameHierarchy:
            description: "Site name hierarchy for the device, must be a child site of\
              \ the discoveredDeviceSiteNameHierarchy or same if it\u2019s not area\
              \ type."
            type: str
        type: list
      discoveryLevel:
        description: Level below primary seed device upto which the new devices will
          be LAN Automated by this session, level + seed = tier. Supported range for
          level is 1-5, default level is 2.
        type: int
      discoveryTimeout:
        description: Discovery timeout in minutes. Until this time, the stop processing
          will not be triggered. Any device contacting after the provided discovery
          timeout will not be processed, and a device reset and reload will be attempted
          to bring it back to the PnP agent state before process completion. The supported
          timeout range is in minutes 20-10080. If both timeout and discovery devices
          list are provided, the stop processing will be attempted whichever happens
          earlier. Users can always use the LAN Automation delete API to force stop
          processing.
        type: int
      hostNameFileId:
        description: Use /dna/intent/api/v1/file/namespace/nw_orch API to get the file
          ID for the already uploaded file in the nw_orch namespace.
        type: str
      hostNamePrefix:
        description: Host name prefix assigned to the discovered device.
        type: str
      ipPools:
        description: Lan Automation V2's ipPools.
        elements: dict
        suboptions:
          ipPoolName:
            description: Name of the IP pool.
            type: str
          ipPoolRole:
            description: Role of the IP pool. Supported roles are MAIN_POOL and PHYSICAL_LINK_POOL.
            type: str
        type: list
      isisDomainPwd:
        description: IS-IS domain password in plain text.
        type: str
      multicastEnabled:
        description: Enable underlay native multicast.
        type: bool
      peerDeviceManagmentIPAddress:
        description: Peer seed management IP address.
        type: str
      primaryDeviceInterfaceNames:
        description: The list of interfaces on primary seed via which the discovered
          devices are connected.
        elements: str
        type: list
      primaryDeviceManagmentIPAddress:
        description: Primary seed management IP address.
        type: str
      redistributeIsisToBgp:
        description: Advertise LAN Automation summary route into BGP.
        type: bool
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for LAN Automation LANAutomationStartV2
  description: Complete reference of the LANAutomationStartV2 API.
  link: https://developer.cisco.com/docs/dna-center/#!l-an-automation-start-v-2
notes:
  - SDK Method used are
    lan_automation.LanAutomation.lan_automation_start_v2,

  - Paths used are
    post /dna/intent/api/v2/lan-automation,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.lan_automation_v2:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    payload:
    - discoveredDeviceSiteNameHierarchy: string
      discoveryDevices:
      - deviceHostName: string
        deviceManagementIPAddress: string
        deviceSerialNumber: string
        deviceSiteNameHierarchy: string
      discoveryLevel: 0
      discoveryTimeout: 0
      hostNameFileId: string
      hostNamePrefix: string
      ipPools:
      - ipPoolName: string
        ipPoolRole: string
      isisDomainPwd: string
      multicastEnabled: true
      peerDeviceManagmentIPAddress: string
      primaryDeviceInterfaceNames:
      - string
      primaryDeviceManagmentIPAddress: string
      redistributeIsisToBgp: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sxp_connections
short_description: Resource module for SXP Connections
description:
- Manage operations create, update and delete of the resource SXP Connections.
- This API creates a SXP connection.
- This API deletes a SXP connection.
- This API allows the client to update a SXP connection.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  description:
    description: SXP Connections's description.
    type: str
  enabled:
    description: Enabled flag.
    type: bool
  id:
    description: SXP Connections's id.
    type: str
  ipAddress:
    description: SXP Connections's ipAddress.
    type: str
  sxpMode:
    description: SXP Connections's sxpMode.
    type: str
  sxpNode:
    description: SXP Connections's sxpNode.
    type: str
  sxpPeer:
    description: SXP Connections's sxpPeer.
    type: str
  sxpVersion:
    description: SXP Connections's sxpVersion.
    type: str
  sxpVpn:
    description: SXP Connections's sxpVpn.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    sxp_connections.SxpConnections.create_sxp_connections,
    sxp_connections.SxpConnections.delete_sxp_connections_by_id,
    sxp_connections.SxpConnections.update_sxp_connections_by_id,

  - Paths used are
    post /ers/config/sxpconnections,
    delete /ers/config/sxpconnections/{id},
    put /ers/config/sxpconnections/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.sxp_connections:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    enabled: true
    id: string
    ipAddress: string
    sxpMode: string
    sxpNode: string
    sxpPeer: string
    sxpVersion: string
    sxpVpn: string

- name: Delete by id
  cisco.ise.sxp_connections:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.sxp_connections:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    enabled: true
    ipAddress: string
    sxpMode: string
    sxpNode: string
    sxpPeer: string
    sxpVersion: string
    sxpVpn: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "description": "string",
      "sxpPeer": "string",
      "sxpVpn": "string",
      "sxpNode": "string",
      "ipAddress": "string",
      "sxpMode": "string",
      "sxpVersion": "string",
      "enabled": true,
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_access_policy_set_reset_hitcount
short_description: Resource module for Network Access Policy Set Reset Hitcount
description:
- Manage operation create of the resource Network Access Policy Set Reset Hitcount.
- Network Access - Reset HitCount for PolicySets.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options: {}
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Network Access - Policy Set
  description: Complete reference of the Network Access - Policy Set API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!policy-openapi
notes:
  - SDK Method used are
    network_access_policy_set.NetworkAccessPolicySet.reset_hit_counts_network_access_policy_sets,

  - Paths used are
    post /network-access/policy-set/reset-hitcount,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.network_access_policy_set_reset_hitcount:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "message": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2017 Obezimnaka Boms, <t-ozboms@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_dnsrecordset_info

version_added: "0.1.2"

short_description: Get DNS Record Set facts

description:
    - Get facts for a specific DNS Record Set in a Zone, or a specific type in all Zones or in one Zone etc.

options:
    relative_name:
        description:
            - Only show results for a Record Set.
        type: str
    resource_group:
        description:
            - Limit results by resource group. Required when filtering by name or type.
        type: str
    zone_name:
        description:
            - Limit results by zones. Required when filtering by name or type.
        type: str
    record_type:
        description:
            - Limit record sets by record type.
        type: str
    top:
        description:
            - Limit the maximum number of record sets to return.
        type: int

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Ozi Boms (@ozboms)

'''

EXAMPLES = '''
- name: Get facts for one Record Set
  azure_rm_dnsrecordset_info:
    resource_group: myResourceGroup
    zone_name: example.com
    relative_name: server10
    record_type: A
- name: Get facts for all Type A Record Sets in a Zone
  azure_rm_dnsrecordset_info:
    resource_group: myResourceGroup
    zone_name: example.com
    record_type: A
- name: Get all record sets in one zone
  azure_rm_dnsrecordset_info:
    resource_group: myResourceGroup
    zone_name: example.com
'''

RETURN = '''
azure_dnsrecordset:
    description:
        - List of record set dicts.
    returned: always
    type: list
    example: [
    {
        "etag": "60ac0480-44dd-4881-a2ed-680d20b3978e",
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/dnszones/newzone.com/A/servera",
        "name": "servera",
        "properties": {
            "ARecords": [
                {
                    "ipv4Address": "10.4.5.7"
                },
                {
                    "ipv4Address": "2.4.5.8"
                }
            ],
            "TTL": 12900
        },
        "type": "Microsoft.Network/dnszones/A"
    }]
dnsrecordsets:
    description:
        - List of record set dicts, which shares the same hierarchy as M(azure.azcollection.azure_rm_dnsrecordset) module's parameter.
    returned: always
    type: complex
    contains:
        id:
            description:
                - ID of the dns recordset.
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/dnszones/newzone.
                     com/A/servera"
        relative_name:
            description:
                - Name of the dns recordset.
            type: str
            sample: servera
        record_type:
            description:
                - The type of the record set.
                - Can be C(A), C(AAAA), C(CNAME), C(MX), C(NS), C(SRV), C(TXT), C(PTR).
            type: str
            sample: A
        time_to_live:
            description:
                - Time to live of the record set in seconds.
            type: int
            sample: 12900
        records:
            description:
                - List of records depending on the type of recordset.
            type: dict
            sample: [
                        {
                            "ipv4Address": "10.4.5.7"
                        },
                        {
                            "ipv4Address": "2.4.5.8"
                        }
                    ]
        provisioning_state:
            description:
                - Provision state of the resource.
            type: str
            sample: Successed
        fqdn:
            description:
                - Fully qualified domain name of the record set.
            type: str
            sample: www.newzone.com
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'RecordSet'


RECORDSET_VALUE_MAP = dict(
    A='a_records',
    AAAA='aaaa_records',
    CNAME='cname_record',
    MX='mx_records',
    NS='ns_records',
    PTR='ptr_records',
    SRV='srv_records',
    TXT='txt_records',
    SOA='soa_record',
    CAA='caa_records'
    # FUTURE: add missing record types from https://github.com/Azure/azure-sdk-for-python/blob/master/azure-mgmt-dns/azure/mgmt/dns/models/record_set.py
)


class AzureRMRecordSetInfo(AzureRMModuleBase):

    def __init__(self):

        # define user inputs into argument
        self.module_arg_spec = dict(
            relative_name=dict(type='str'),
            resource_group=dict(type='str'),
            zone_name=dict(type='str'),
            record_type=dict(type='str'),
            top=dict(type='int')
        )

        # store the results of the module operation
        self.results = dict(
            changed=False,
        )

        self.relative_name = None
        self.resource_group = None
        self.zone_name = None
        self.record_type = None
        self.top = None

        super(AzureRMRecordSetInfo, self).__init__(self.module_arg_spec, supports_check_mode=True)

    def exec_module(self, **kwargs):

        is_old_facts = self.module._name == 'azure_rm_dnsrecordset_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_dnsrecordset_facts' module has been renamed to 'azure_rm_dnsrecordset_info'", version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if not self.top or self.top <= 0:
            self.top = None

        # create conditionals to catch errors when calling record facts
        if self.relative_name and not self.resource_group:
            self.fail("Parameter error: resource group required when filtering by name or record type.")
        if self.relative_name and not self.zone_name:
            self.fail("Parameter error: DNS Zone required when filtering by name or record type.")

        results = []
        # list the conditions for what to return based on input
        if self.relative_name is not None:
            # if there is a name listed, they want only facts about that specific Record Set itself
            results = self.get_item()
        elif self.record_type:
            # else, they just want all the record sets of a specific type
            results = self.list_type()
        elif self.zone_name:
            # if there is a zone name listed, then they want all the record sets in a zone
            results = self.list_zone()

        if is_old_facts:
            self.results['ansible_facts'] = {
                'azure_dnsrecordset': self.serialize_list(results)
            }
        self.results['dnsrecordsets'] = self.curated_list(results)
        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.relative_name))
        item = None
        results = []

        # try to get information for specific Record Set
        try:
            item = self.dns_client.record_sets.get(self.resource_group, self.zone_name, self.relative_name, self.record_type)
        except ResourceNotFoundError:
            results = []
            pass
        else:
            results = [item]
        return results

    def list_type(self):
        self.log('Lists the record sets of a specified type in a DNS zone')
        try:
            response = self.dns_client.record_sets.list_by_type(self.resource_group, self.zone_name, self.record_type, top=self.top)
        except Exception as exc:
            self.fail("Failed to list for record type {0} - {1}".format(self.record_type, str(exc)))

        results = []
        for item in response:
            results.append(item)
        return results

    def list_zone(self):
        self.log('Lists all record sets in a DNS zone')
        try:
            response = self.dns_client.record_sets.list_by_dns_zone(self.resource_group, self.zone_name, top=self.top)
        except Exception as exc:
            self.fail("Failed to list for zone {0} - {1}".format(self.zone_name, str(exc)))

        results = []
        for item in response:
            results.append(item)
        return results

    def serialize_list(self, raws):
        return [self.serialize_obj(item, AZURE_OBJECT_CLASS) for item in raws] if raws else []

    def curated_list(self, raws):
        return [self.record_to_dict(item) for item in raws] if raws else []

    def record_to_dict(self, record):
        record_type = record.type[len('Microsoft.Network/dnszones/'):]
        records = getattr(record, RECORDSET_VALUE_MAP.get(record_type))
        if records:
            if not isinstance(records, list):
                records = [records]
        else:
            records = []
        return dict(
            id=record.id,
            relative_name=record.name,
            record_type=record_type,
            records=[x.as_dict() for x in records],
            time_to_live=record.ttl,
            fqdn=record.fqdn,
            provisioning_state=record.provisioning_state,
            metadata=record.metadata
        )


def main():
    AzureRMRecordSetInfo()


if __name__ == '__main__':
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.conf.models import Setting


@pytest.mark.django_db
def test_setting_flat_value(run_module, admin_user):
    the_value = 'CN=service_account,OU=ServiceAccounts,DC=domain,DC=company,DC=org'
    result = run_module('settings', dict(name='AUTH_LDAP_BIND_DN', value=the_value), admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result

    assert Setting.objects.get(key='AUTH_LDAP_BIND_DN').value == the_value


@pytest.mark.django_db
def test_setting_dict_value(run_module, admin_user):
    the_value = {'email': 'mail', 'first_name': 'givenName', 'last_name': 'surname'}
    result = run_module('settings', dict(name='AUTH_LDAP_USER_ATTR_MAP', value=the_value), admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result

    assert Setting.objects.get(key='AUTH_LDAP_USER_ATTR_MAP').value == the_value


@pytest.mark.django_db
def test_setting_nested_type(run_module, admin_user):
    the_value = {'email': 'mail', 'first_name': 'givenName', 'last_name': 'surname'}
    result = run_module('settings', dict(settings={'AUTH_LDAP_USER_ATTR_MAP': the_value}), admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed'), result

    assert Setting.objects.get(key='AUTH_LDAP_USER_ATTR_MAP').value == the_value


@pytest.mark.django_db
def test_setting_bool_value(run_module, admin_user):
    for the_value in (True, False):
        result = run_module('settings', dict(name='ACTIVITY_STREAM_ENABLED_FOR_INVENTORY_SYNC', value=the_value), admin_user)
        assert not result.get('failed', False), result.get('msg', result)
        assert result.get('changed'), result

        assert Setting.objects.get(key='ACTIVITY_STREAM_ENABLED_FOR_INVENTORY_SYNC').value is the_value



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: http_write_credential
short_description: Resource module for Http Write Credential
description:
- This module represents an alias of the module http_write_credential_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  comments:
    description: Comments to identify the HTTP(S) Write credential.
    type: str
  credentialType:
    description: Credential type to identify the application that uses the HTTP(S) Write
      credential.
    type: str
  description:
    description: Description for HTTP(S) Write Credential.
    type: str
  id:
    description: Id of the HTTP(S) Write Credential in UUID format.
    type: str
  instanceTenantId:
    description: Deprecated.
    type: str
  instanceUuid:
    description: Deprecated.
    type: str
  password:
    description: HTTP(S) Write Password.
    type: str
  port:
    description: HTTP(S) Port. Valid port should be in the range of 1 to 65535.
    type: int
  secure:
    description: Flag for HTTPS Write.
    type: bool
  username:
    description: HTTP(S) Write Username.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Discovery CreateHTTPWriteCredentialsV1
  description: Complete reference of the CreateHTTPWriteCredentialsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!create-http-write-credentials
- name: Cisco DNA Center documentation for Discovery UpdateHTTPWriteCredentialsV1
  description: Complete reference of the UpdateHTTPWriteCredentialsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-http-write-credentials
notes:
  - SDK Method used are
    discovery.Discovery.create_http_write_credentials_v1,
    discovery.Discovery.update_http_write_credentials_v1,

  - Paths used are
    post /dna/intent/api/v1/global-credential/http-write,
    put /dna/intent/api/v1/global-credential/http-write,
  - It should be noted that this module is an alias of http_write_credential_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.http_write_credential:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    comments: string
    credentialType: string
    description: string
    id: string
    instanceTenantId: string
    instanceUuid: string
    password: string
    port: 0
    secure: true
    username: string

- name: Create
  cisco.dnac.http_write_credential:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    comments: string
    credentialType: string
    description: string
    id: string
    instanceTenantId: string
    instanceUuid: string
    password: string
    port: 0
    secure: true
    username: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: global_credential_v1_info
short_description: Information module for Global Credential V1
description:
- Get all Global Credential V1.
- Get Global Credential V1 by id.
- Returns global credential for the given credential sub type.
- Returns the credential sub type for the given Id.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  credentialSubType:
    description:
    - >
      CredentialSubType query parameter. Credential type as CLI / SNMPV2_READ_COMMUNITY / SNMPV2_WRITE_COMMUNITY /
      SNMPV3 / HTTP_WRITE / HTTP_READ / NETCONF.
    type: str
  sortBy:
    description:
    - SortBy query parameter. Field to sort the results by. Sorts by 'instanceId' if no value is provided.
    type: str
  order:
    description:
    - Order query parameter. Order of sorting. 'asc' or 'des'.
    type: str
  id:
    description:
    - Id path parameter. Global Credential ID.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Discovery GetCredentialSubTypeByCredentialIdV1
  description: Complete reference of the GetCredentialSubTypeByCredentialIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-credential-sub-type-by-credential-id
- name: Cisco DNA Center documentation for Discovery GetGlobalCredentialsV1
  description: Complete reference of the GetGlobalCredentialsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-global-credentials
notes:
  - SDK Method used are
    discovery.Discovery.get_credential_sub_type_by_credential_id_v1,
    discovery.Discovery.get_global_credentials_v1,

  - Paths used are
    get /dna/intent/api/v1/global-credential,
    get /dna/intent/api/v1/global-credential/{id},

"""

EXAMPLES = r"""
- name: Get all Global Credential V1
  cisco.dnac.global_credential_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    credentialSubType: string
    sortBy: string
    order: string
  register: result

- name: Get Global Credential V1 by id
  cisco.dnac.global_credential_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": "string",
      "version": "string"
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr lldp_interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.lldp_interfaces.lldp_interfaces import (
    Lldp_interfacesArgs,
)


class Lldp_interfacesFacts(object):
    """The iosxr lldp_interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Lldp_interfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_config(self, connection):
        return connection.get_config(flags="interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for lldp_interfaces
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        if not data:
            data = self.get_config(connection)
        interfaces = ("\n" + data).split("\ninterface ")

        objs = []
        for interface in interfaces:
            obj = self.render_config(self.generated_spec, interface)
            if obj:
                objs.append(obj)

        ansible_facts["ansible_network_resources"].pop("lldp_interfaces", None)
        facts = {}

        if objs:
            facts["lldp_interfaces"] = []
            params = utils.validate_config(
                self.argument_spec,
                {"config": objs},
            )
            for cfg in params["config"]:
                facts["lldp_interfaces"].append(utils.remove_empties(cfg))

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        match = re.search(
            r"(GigabitEthernet|Bundle-Ether|TenGigE|FortyGigE|HundredGigE)(\S+)",
            conf,
            re.M,
        )
        if match:
            config["name"] = match.group(1) + match.group(2)

            for key in ["receive", "transmit"]:
                config[key] = False if ("{0} disable".format(key)) in conf else None

            for x in ["ieee-nearest-bridge", "ieee-nearest-non-tmpr-bridge"]:
                if x in conf:
                    config["destination"]["mac_address"] = x

        return utils.remove_empties(config)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: px_grid_direct_sync_info
short_description: Information module for Px Grid Direct Sync
description:
- Get Px Grid Direct Sync by name.
- This API is used to get the status for SyncNow Status.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  connectorName:
    description:
    - ConnectorName path parameter. Retrieve the connector syncnow status.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    px_grid_direct.PxGridDirect.get_connector_config_sync_now_status,

  - Paths used are
    get /api/v1/pxgrid-direct/syncNowStatus/{connectorName},

"""

EXAMPLES = r"""
- name: Get Px Grid Direct Sync by name
  cisco.ise.px_grid_direct_sync_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    connectorName: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "connector": {
        "connectorName": "string",
        "syncStatus": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: node_group
short_description: Resource module for Node Group
description:
- Manage operations create, update and delete of the resource Node Group.
- This API creates a node group in the cluster. A node group is a group of PSNs,.
- Delete an existing node group in the cluster. Deleting the node group does not delete the nodes, but failover is no longer carried out among the nodes.
- Update an existing node group.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  description:
    description: Node Group's description.
    type: str
  forceDelete:
    description: ForceDelete query parameter. Force delete the group even if the node
      group contains one or more nodes.
    type: bool
  marCache:
    description: Node Group's marCache.
    suboptions:
      query-attempts:
        description: The number of times Cisco ISE attempts to perform the cache entry
          query. (0 - 5).
        type: int
      query-timeout:
        description: The time, in seconds, after which the cache entry query times out.
          (1 - 10).
        type: int
      replication-attempts:
        description: The number of times Cisco ISE attempts to perform MAR cache entry
          replication. (0 - 5).
        type: int
      replication-timeout:
        description: The time, in seconds, after which the cache entry replication times
          out. (1 - 10).
        type: int
    type: dict
  name:
    description: Node Group's name.
    type: str
  nodeGroupName:
    description: NodeGroupName path parameter. Name of the existing node group.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Node Group
  description: Complete reference of the Node Group API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!deployment-openapi
notes:
  - SDK Method used are
    node_group.NodeGroup.create_node_group,
    node_group.NodeGroup.delete_node_group,
    node_group.NodeGroup.update_node_group,

  - Paths used are
    post /api/v1/deployment/node-group,
    delete /api/v1/deployment/node-group/{nodeGroupName},
    put /api/v1/deployment/node-group/{nodeGroupName},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.node_group:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    marCache:
      query-attempts: 0
      query-timeout: 0
      replication-attempts: 0
      replication-timeout: 0
    name: string

- name: Update by name
  cisco.ise.node_group:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    description: string
    marCache:
      query-attempts: 0
      query-timeout: 0
      replication-attempts: 0
      replication-timeout: 0
    name: string
    nodeGroupName: string

- name: Delete by name
  cisco.ise.node_group:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    forceDelete: true
    nodeGroupName: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "name": "string",
      "description": "string",
      "marCache": {
        "replication-timeout": 0,
        "replication-attempts": 0,
        "query-timeout": 0,
        "query-attempts": 0
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "success": {
        "message": "string"
      },
      "version": "string"
    }
"""
